/* display.c, X11 interface                                                 */

/*
 * All modifications (mpeg2decode -> mpeg2play) are
 * Copyright (C) 1994, Stefan Eckart. All Rights Reserved.
 */

/* XvPutImage modifications by Amancio Hasty -- hasty@star-gate.com */

/* Copyright (C) 1994, MPEG Software Simulation Group. All Rights Reserved. */

/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis.  The MPEG Software Simulation Group disclaims
 * any and all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and user's
 * customers, employees, agents, transferees, successors, and assigns.
 *
 * The MPEG Software Simulation Group does not represent or warrant that the
 * programs furnished hereunder are free of infringement of any third-party
 * patents.
 *
 * Commercial implementations of MPEG-1 and MPEG-2 video, including shareware,
 * are subject to royalty fees to patent holders.  Many of these patents are
 * general enough such that they are unavoidable regardless of implementation
 * design.
 *
 */

 /* the Xlib interface is closely modeled after
  * mpeg_play 2.0 by the Berkeley Plateau Research Group
  */

#include <stdio.h>
#include <stdlib.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/extensions/Xvlib.h>
#include <signal.h>
#include "config.h"
#include "global.h"

/* private prototypes */
static void display_image _ANSI_ARGS_((XvImage *ximage, unsigned char *dithered_image));
static void ditherframe _ANSI_ARGS_((unsigned char *src[]));

/* local data */
static unsigned char *dithered_image, *dithered_image2;



/* X11 related variables */
static Display *display;
static Window window;
static GC gc;
static XvImage *ximage, *ximage2;
static unsigned char pixel[256];

#ifdef SH_MEM

#include <sys/ipc.h>
#include <sys/shm.h>
#include <X11/extensions/XShm.h>
#include <X11/extensions/Xvlib.h>

static int HandleXError _ANSI_ARGS_((Display *dpy, XErrorEvent *event));
static void InstallXErrorHandler _ANSI_ARGS_((void));
static void DeInstallXErrorHandler _ANSI_ARGS_((void));

static int shmem_flag;
static XShmSegmentInfo shminfo1, shminfo2;
static int gXErrorFlag;
static int CompletionType = -1;

static hheight = 0;
static hwidth = 0;

/* Xv */
#define GUID_YUV12_PLANAR  0x32315659 
 XvAdaptorInfo *info;
 XvImage *yuv_image;
 XvImageFormatValues *formats;
 int yuv_height=0;
 int yuv_width=0;

 int numAdaptors = 2;
 int adaptor = -1;

/* end of Xv */
static int HandleXError(dpy, event)
Display *dpy;
XErrorEvent *event;
{
  gXErrorFlag = 1;

  return 0;
}

static void InstallXErrorHandler()
{
  XSetErrorHandler(HandleXError);
  XFlush(display);
}

static void DeInstallXErrorHandler()
{
  XSetErrorHandler(NULL);
  XFlush(display);
}

#endif

/* connect to server, create and map window,
 * allocate colors and (shared) memory
 */
void init_display(name)
char *name;
{
  int crv, cbu, cgu, cgv;
  int y, u, v, r, g, b;
  int i,j,num;
  char dummy;
  int screen;


  unsigned int fg, bg;
  char *hello = "MPEG-2 Display";
  XSizeHints hint;
  XVisualInfo vinfo;
  XEvent xev;
  unsigned long tmp_pixel;
  XWindowAttributes xwa;
  XTextProperty windowName, iconName;
  char xv_name[6];
  display = XOpenDisplay(name);

  if (display == NULL)
    error("Can not open display\n");

  screen = DefaultScreen(display);

  hint.x = 200;
  hint.y = 200;
  hint.width = horizontal_size;
  hint.height = vertical_size;
  hint.flags = PPosition | PSize;

  /* Get some colors */

  bg = WhitePixel (display, screen);
  fg = BlackPixel (display, screen);

  /* Make the window */


  window = XCreateSimpleWindow (display, DefaultRootWindow (display),
             hint.x, hint.y, hint.width, hint.height, 4, fg, bg);

  hwidth = hint.width;
  hheight = hint.height;

  XSelectInput(display, window, StructureNotifyMask);
  /* Tell other applications about this window */
  XSetStandardProperties (display, window, hello, hello, None, NULL, 0, &hint);

  /* Map window. */

  XMapWindow(display, window);

  /* Wait for map. */
  do
  {
    XNextEvent(display, &xev);
  }
  while (xev.type != MapNotify || xev.xmap.event != window);

  /*  XSelectInput(display, window, NoEventMask); */



  /* allocate colors */

  gc = DefaultGC(display, screen);




  yuv_height = coded_picture_height;
  yuv_width = coded_picture_width;

  if (adaptor < 0 ) {
    adaptor = -1;
    xv_name[4]=0;

    XvQueryAdaptors(display, DefaultRootWindow(display), &numAdaptors, &info);
      for(i = 0; i < numAdaptors; i++) {
	num = 0;
	formats  = XvListImageFormats(display, info[i].base_id, &num);
	for(j = 0; j < num; j++) {
	  fprintf(stderr, "   0x%x ", formats[j].id);
	  memcpy(xv_name, &formats[j].id, 4);
	  fprintf(stderr, "(%s) %s\n", xv_name, (formats[j].format == XvPacked) ?
					"packed" : "planar");
	  if (formats[j].id ==  GUID_YUV12_PLANAR) {
	    adaptor = i;
	  }
	}
      }

      if (adaptor == -1 ) {
	printf("no suitable Xv adaptor found");
	exit(1);
      }


      CompletionType = XShmGetEventBase(display) + ShmCompletion;


      ximage = (XvImage *)  XvShmCreateImage(display, info[adaptor].base_id, 
				GUID_YUV12_PLANAR,0, yuv_width, yuv_height, (char *) &shminfo1);
      shminfo1.shmid = shmget (IPC_PRIVATE, ximage->data_size, IPC_CREAT|0777);
      shminfo1.shmaddr = ximage->data = shmat (shminfo1.shmid, 0, 0);
      shminfo1.readOnly = False;
      if(!XShmAttach (display, &shminfo1)) {
	fprintf(stderr, "XShmAttach failed!\n");
      }

      if (!prog_seq) {
	ximage2 = (XvImage *)  XvShmCreateImage(display, info[adaptor].base_id, 
				GUID_YUV12_PLANAR,0, yuv_width, yuv_height, (char *) &shminfo2);
	shminfo2.shmid = shmget (IPC_PRIVATE, ximage2->data_size, IPC_CREAT|0777);
	shminfo2.shmaddr = ximage2->data = shmat (shminfo2.shmid, 0, 0);
	shminfo2.readOnly = False;
	if(!XShmAttach (display, &shminfo2)) {
	  fprintf(stderr, "XShmAttach failed!\n");
	}
      }


      signal(SIGTERM, (sig_t) exit_display);
      signal(SIGINT, (sig_t) exit_display);
      XSync(display,False);
  }
  /* End XV */




  DeInstallXErrorHandler();


}



void exit_display()
{

    XShmDetach(display, &shminfo1);
    /*    XDestroyImage(ximage); */
    shmdt(shminfo1.shmaddr);
    shmctl (shminfo1.shmid, IPC_RMID, 0);
    if (!prog_seq)
    {
      XShmDetach(display, &shminfo2);
      /*      XDestroyImage(ximage2); */
      shmctl (shminfo2.shmid, IPC_RMID, 0);
      shmdt(shminfo2.shmaddr);
    }
    exit(0);
}

static void display_image(ximage,dithered_image)
XvImage *ximage;
unsigned char *dithered_image;
{
  /* display dithered image */
  int _w, _h, _d;
  Window _dw;

  XvShmPutImage(display, info[adaptor].base_id,
		      window ,
		      gc,  ximage, 0, 0, 
		      yuv_width, yuv_height, 0, 0, 
		      hwidth, hheight, True);


      
  while (1)
    {
      XEvent xev;
	
      XNextEvent(display, &xev);
      if (xev.type == CompletionType)
        break;

      if (xev.type == ConfigureNotify) {
	XGetGeometry(display, window, &_dw, &_d, &_d, &hwidth, &hheight, &_d, &_d); 
      }
    }

}

void display_second_field()
{
  display_image(ximage2,dithered_image2);
}



void dither(src)
unsigned char *src[];
{

  bcopy(src[0], ximage->data, ximage->data_size);
  display_image(ximage,dithered_image);
  return;


}



