#include "Xarch.h"
#include "xaa.h"
#include "xf86.h"
#include "xf86_OSproc.h"
#include "xf86_ansic.h"
#include "xf86Pci.h"
#include "xf86PciInfo.h"
#include "xf86Xinput.h"
#include "xf86fbman.h"
#include "xf86i2c.h"
#include "xf86xv.h"
#include "Xv.h"
#include "xf86Cursor.h"
#include "regionstr.h"
#include "xaalocal.h"

#include "r128.h"
#include "r128_reg.h"
#include "r128_video.h"
#include "r128_i2c.h"
#include "bt829.h"

#define LIMIT(X,A,B)  (((X)<(A)) ? (A) : ((X)>(B)) ? (B) : (X) )


/* Bt829 family chip ID's */
#define BT815	0x02
#define BT817	0x06
#define BT819	0x07
#define BT827	0x0C
#define BT829	0x0E

/* Bt829 registers */
#define STATUS		0x00	/* Device Status */
#define IFORM		0x01	/* Input Format */
#define TDEC		0x02	/* Temporal Decimation */
#define CROP		0x03	/* MSB Cropping */
#define VDELAY_LO	0x04	/* Vertical Delay */
#define VACTIVE_LO	0x05	/* Vertical Active */
#define HDELAY_LO	0x06	/* Horizontal Delay */
#define HACTIVE_LO	0x07	/* Horizontal Active */
#define HSCALE_HI	0x08	/* Horizontal Scaling */
#define HSCALE_LO	0x09	/* Horizontal Scaling */
#define BRIGHT		0x0A	/* Brightness Control */
#define CONTROL		0x0B	/* Miscellaneous Control */
#define CONTRAST_LO	0x0C	/* Luma Gain (Contrast) */
#define SAT_U_LO	0x0D	/* Chroma (U) Gain (Saturation) */
#define SAT_V_LO	0x0E	/* Chroma (V) Gain (Saturation) */
#define HUE		0x0F	/* Hue Control */
#define SCLOOP		0x10	/* SC Loop Control */
#define WC_UP		0x11	/* White Crush Up Count */
#define OFORM		0x12	/* Output Format */
#define VSCALE_HI	0x13	/* Vertical Scaling */
#define VSCALE_LO	0x14	/* Vertical Scaling */
#define TEST		0x15	/* Test Control */
#define VPOLE		0x16	/* Video Timing Polarity */
#define IDCODE		0x17	/* ID Code */
#define ADELAY		0x18	/* AGC Delay */
#define BDELAY		0x19	/* Burst Gate Delay */
#define ADC		0x1A	/* ADC Interface */
#define VTC		0x1B	/* Video Timing Control */
#define CC_STATUS	0x1C	/* Extended Data Services/Closed Capt Status */
#define CC_DATA		0x1D	/* Extended Data Services/Closed Capt Data */
#define WC_DN		0x1E	/* White Crush Down Count */
#define SRESET		0x1F	/* Software Reset */
#define P_IO		0x3F	/* Programmable I/O */


CARD8 btread(PortPrivPtr pPPriv, CARD8 reg)
{
I2C_PACKET p;
CARD8 v;

p.number_of_bytes=1;
p.address=pPPriv->bt829_addr;
p.data=&reg;
p.start=1;
p.stop=1;
I2C_Set(pPPriv,&p);
p.number_of_bytes=1;
p.address=pPPriv->bt829_addr;
p.data=&v;
p.start=1;
p.stop=1;
I2C_Get(pPPriv,&p);
return v;
}

void btwrite(PortPrivPtr pPPriv, CARD8 reg, CARD8 val)
{
I2C_PACKET p;
CARD8 data[2];
CARD8 retval;
CARD8 counter;

p.number_of_bytes=2;
p.address=pPPriv->bt829_addr;
p.data=data;
data[0]=reg;
data[1]=val;
p.start=1;
p.stop=1;
counter=0;
do{
	retval=I2C_Set(pPPriv, &p);
	counter++;
  } while ((counter<p.number_of_bytes)&&(retval!=I2C_DONE));
}

int bt829_init(PortPrivPtr pPPriv) 
{
AdaptorPrivPtr pAPriv = pPPriv->pAdaptor;
ScrnInfoPtr	pScrn=pAPriv->pScrn;


pPPriv->bt829_xtsel=0;
pPPriv->bt829_htotal=0; 
pPPriv->bt829_vtotal=0; 
pPPriv->bt829_ldec=1; 
pPPriv->bt829_cbsense=0;
pPPriv->bt829_adelay=0; 
pPPriv->bt829_bdelay=0; 
pPPriv->bt829_hdelay=0; 
pPPriv->bt829_vdelay=22;
pPPriv->bt829_luma=0; 
pPPriv->bt829_sat_u=0; 
pPPriv->bt829_sat_v=0;



  pPPriv->bt829_addr=0;
  
  if(I2CProbeAddress(pPPriv,0x88))pPPriv->bt829_addr=0x88;
  if(I2CProbeAddress(pPPriv,0x8a))pPPriv->bt829_addr=0x8a;

  if(!pPPriv->bt829_addr){
  	return pPPriv->bt829_addr;
  	}

  pPPriv->bt829_id=BTREAD(IDCODE);

  switch((pPPriv->bt829_id)>>4){
  	case BT815:
		xf86DrvMsg(pScrn->scrnIndex,X_INFO,"Found bt815a video decoder, revision %d\n",pPPriv->bt829_id & 0xf);
		break;
	case BT817:
		xf86DrvMsg(pScrn->scrnIndex,X_INFO,"Found bt817a video decoder, revision %d\n",pPPriv->bt829_id & 0xf);
  		break;
	case BT819:
		xf86DrvMsg(pScrn->scrnIndex,X_INFO,"Found bt819a video decoder, revision %d\n",pPPriv->bt829_id & 0xf);
  		break;
	case BT827:
		xf86DrvMsg(pScrn->scrnIndex,X_INFO,"Found bt827a/b video decoder, revision %d\n",pPPriv->bt829_id & 0xf);
  		break;
	case BT829:
		xf86DrvMsg(pScrn->scrnIndex,X_INFO,"Found bt829a/b video decoder, revision %d\n",pPPriv->bt829_id & 0xf);
  		break;
	default:
		xf86DrvMsg(pScrn->scrnIndex,X_INFO,"Found bt8xx/unknown video decoder version %d, revision %d\n",pPPriv->bt829_id >> 4,pPPriv->bt829_id & 0xf);
  		break;
	}
	
  pPPriv->bt829_vpole=1; /* all-in-wonder 128 uses vpole=1 */
  	
  BTWRITE(SRESET,0x0);
  BTWRITE(VPOLE,(pPPriv->bt829_vpole)<<7) ;

  return pPPriv->bt829_addr;  
}

void bt829_setformat(PortPrivPtr pPPriv, int format) {
  int tunertype=1; /* fix this later */

  /* NTSC, PAL or SECAM ? */
  pPPriv->bt829_format=format;
  switch (pPPriv->bt829_format) {
    case 2: case 4:						/* NTSC */
      if ((pPPriv->bt829_id>>4) <= BT819) return; /* can't do it */
    case 1:							/* NTSC */
      pPPriv->bt829_adelay = 104 ; 
      pPPriv->bt829_bdelay = 93 ; 
      pPPriv->bt829_xtsel = 1 ; 
      pPPriv->bt829_vdelay = 22 ;
      pPPriv->bt829_htotal = 754 ; 
      pPPriv->bt829_vtotal = 480 ; 
      break ;
    case 5: case 7:						/* PAL */
      if ((pPPriv->bt829_id>>4) <= BT819) return ; /* can't do it */
    case 3:							/* PAL */
      pPPriv->bt829_vdelay = (tunertype==5) ? 34 : 22 ;
      pPPriv->bt829_adelay = 127 ; 
      pPPriv->bt829_bdelay = 114 ; 
      pPPriv->bt829_xtsel = 2 ;
/*      pPPriv->bt829_htotal = 888 ;  */
	pPPriv->bt829_htotal=922;  /* according to Marko..*/
      pPPriv->bt829_vtotal = 576 ; 
      break ;
    case 6:							/* SECAM */
      if ((pPPriv->bt829_id>>4) <= BT819) return; /* can't do it */
      pPPriv->bt829_adelay = 127 ; 
      pPPriv->bt829_bdelay = 160 ; 
      pPPriv->bt829_xtsel = 2 ; 
      pPPriv->bt829_vdelay = 34 ;
/*      pPPriv->bt829_htotal = 888 ; */
      pPPriv->bt829_htotal=922;  /* according to Marko..*/
      pPPriv->bt829_vtotal = 576 ; 
      break ; 
    }

  /* Program the Bt829 */
  bt829_iform(pPPriv) ;
  BTWRITE(TDEC,0x00) ;
  BTWRITE(VDELAY_LO,L(pPPriv->bt829_vdelay)) ;
  BTWRITE(OFORM,0x0A) ;
  BTWRITE(ADELAY,pPPriv->bt829_adelay) ;
  BTWRITE(BDELAY,pPPriv->bt829_bdelay) ;
  BTWRITE(ADC,0x82) ;
  if ((pPPriv->bt829_id>>4) >= BT827) {
    BTWRITE(SCLOOP,(format==6)?0x10:0x00) ;
    BTWRITE(WC_UP,0xCF) ;
    bt829_setCC(pPPriv) ;
    BTWRITE(WC_DN,0x7F) ;
    BTWRITE(P_IO,0x00) ; }
  bt829_setmux(pPPriv) ;
  bt829_SetBrightness(pPPriv,pPPriv->iBrightness) ;
  bt829_SetContrast(pPPriv,pPPriv->iContrast) ;
  bt829_SetSaturation(pPPriv,pPPriv->iSaturation) ;
  bt829_SetTint(pPPriv,pPPriv->iHue) ;
}

/* ------------------------------------------------------------------------ */
/* Public routines */

void bt829_setmux(PortPrivPtr pPPriv) 
{ 
if(pPPriv->bt829_mux==2){
	btwrite(pPPriv,P_IO,1);
	}
bt829_ctrl(pPPriv) ; 
bt829_iform(pPPriv) ; 
}

void bt829_SetBrightness(PortPrivPtr pPPriv, int b) 
{
  pPPriv->iBrightness=b; 
  b = (127*(b))/1000 ;
  b = LIMIT(b,-128,127) ; 
  BTWRITE(BRIGHT,b) ;; 
}

void bt829_SetContrast(PortPrivPtr pPPriv, int c) 
{
  pPPriv->iContrast=c;
  c = (216*(c+1000))/1000 ; 
  pPPriv->bt829_luma = LIMIT(c,0,511) ;
  bt829_ctrl(pPPriv) ; 
  BTWRITE(CONTRAST_LO,L(pPPriv->bt829_luma)) ; 
}

void bt829_SetSaturation(PortPrivPtr pPPriv, int s)
{
  pPPriv->iSaturation=s;
  s=(s+1000)/10;
  s = LIMIT(s, 0, 200);
  pPPriv->bt829_sat_u = 254*s/100 ; pPPriv->bt829_sat_v = 180*s/100 ;
  bt829_ctrl(pPPriv) ; 
  BTWRITE(SAT_U_LO,L(pPPriv->bt829_sat_u)) ; 
  BTWRITE(SAT_V_LO,L(pPPriv->bt829_sat_v)) ;
}

void bt829_SetTint(PortPrivPtr pPPriv, int h) 
{
  pPPriv->iHue=h;
  h = 128*h/1000 ; h = LIMIT(h,-128,127) ;
  BTWRITE(HUE,h) ;   
}

void bt829_setcaptsize(PortPrivPtr pPPriv) 
{

  CARD16 hscale, vscale ; CARD8 yci=0x60, vfilt=0 ;

  /* NTSC, PAL or SECAM ? */
  switch (pPPriv->bt829_format) {
    case 1: case 2: case 4:				/* NTSC */
      pPPriv->bt829_hdelay = pPPriv->overlay_width*135/754 ;
      if (pPPriv->overlay_width <= 240) vfilt = 1 ;
      if (pPPriv->overlay_width <= 120) vfilt = 2 ;
      if (pPPriv->overlay_width <=  60) vfilt = 3 ; break ;
    case 3: case 5: case 6: case 7:			/* PAL/SECAM */
/*      pPPriv->bt829_hdelay = pPPriv->overlay_width*176/768 ;
	Marko says should be 186/922 */
      pPPriv->bt829_hdelay = pPPriv->overlay_width*186/922 ;
      if (pPPriv->overlay_width <= 384) vfilt = 1 ;
      if (pPPriv->overlay_width <= 192) vfilt = 2 ;
      if (pPPriv->overlay_width<=  96) vfilt = 3 ; break ;
    default: return ; /* can't do it */ }

  pPPriv->bt829_ldec = (pPPriv->overlay_width> 384) ;
  pPPriv->bt829_cbsense = pPPriv->bt829_hdelay & 1 ;
  hscale = 4096*pPPriv->bt829_htotal/pPPriv->overlay_width-4096 ;
  vscale = (0x10000 - (512*pPPriv->bt829_vtotal/pPPriv->overlay_height-512)) & 0x1FFF ;

  bt829_crop(pPPriv) ; bt829_ctrl(pPPriv) ;
  BTWRITE(VACTIVE_LO,L(pPPriv->bt829_vtotal)) ;
  BTWRITE(HDELAY_LO,L(pPPriv->bt829_hdelay)) ;
  BTWRITE(HACTIVE_LO,L(pPPriv->overlay_width)) ;
  BTWRITE(HSCALE_HI,H(hscale)) ;
  BTWRITE(HSCALE_LO,L(hscale)) ;
  BTWRITE(VSCALE_HI,H(vscale)|yci) ;
  BTWRITE(VSCALE_LO,L(vscale)) ;
  if ((pPPriv->bt829_id>>4) >= BT827) BTWRITE(VTC,0x18|vfilt) ;

}

int bt829_setCC(PortPrivPtr pPPriv) 
{
  if ((pPPriv->bt829_id >>4) < BT827) {
    pPPriv->CCmode = 0;
    return -1; }
  if (pPPriv->CCmode == 0) { /* Powering off circuitry */
    BTWRITE(CC_STATUS,0x00);
     return 0; }
  /* 0x40 is activate to set the CCVALID line. Not required yet */
  BTWRITE(CC_STATUS,(pPPriv->CCmode<<4)|0x40);
/* we write to STATUS to reset the CCVALID flag */
  BTWRITE(STATUS,0x00);
  return 0; 
}

#if 0

void bt829_getCCdata(PortPrivPtr pPPriv,struct CCdata *data)
{
  CARD8 status;
  data->num_valid=0;
  /* wait for buffer to be half full (means 8/16 bytes)
   * either 4 (one of CC/EDS) or 2 (both CC/EDS) frames */
  if(!(BTREAD(STATUS)&0x04)) return; /* could comment this line */
  for(;data->num_valid<CC_FIFO_SIZE;data->num_valid++) {
    status=BTREAD(CC_STATUS);
    if(!(status&0x04)) break;
    data->data[data->num_valid]= BTREAD(CC_DATA)&0x7f;
                         /* stripped high bit (parity) */
    data->status[data->num_valid]= (CCS_EDS*((status&0x02)>>1))  |
                                 (CCS_HIGH*(status&0x01)) |
                                 (CCS_OVER*((status&0x08)>>3)) |
                                 (CCS_PAR*((status&0x80)>>7)) ; }
  BTWRITE(STATUS,0x00); /* Reset CCVALID status bit */
  return;
}

#endif

/* ------------------------------------------------------------------------ */
/* Debug and report routines */

#define DUMPREG(REG)   \
  xf86DrvMsg(pScrn->scrnIndex,X_INFO," %-12s (0x%02X) = 0x%02X\n", \
              #REG,REG,BTREAD(REG))

void bt829_dumpregs(PortPrivPtr pPPriv) 
{
AdaptorPrivPtr pAPriv=pPPriv->pAdaptor;
ScrnInfoPtr pScrn=pAPriv->pScrn;

  DUMPREG(STATUS) ;
  DUMPREG(IFORM) ;
  DUMPREG(TDEC) ;
  DUMPREG(CROP) ;
  DUMPREG(VDELAY_LO) ;
  DUMPREG(VACTIVE_LO) ;
  DUMPREG(HDELAY_LO) ;
  DUMPREG(HACTIVE_LO) ;
  DUMPREG(HSCALE_HI) ;
  DUMPREG(HSCALE_LO) ;
  DUMPREG(BRIGHT) ;
  DUMPREG(CONTROL) ;
  DUMPREG(CONTRAST_LO) ;
  DUMPREG(SAT_U_LO) ;
  DUMPREG(SAT_V_LO) ;
  DUMPREG(HUE) ;
  if ((pPPriv->bt829_id>>4) >= BT827) {
    DUMPREG(SCLOOP) ;
    DUMPREG(WC_UP) ; }
  DUMPREG(OFORM) ;
  DUMPREG(VSCALE_HI) ;
  DUMPREG(VSCALE_LO) ;
  DUMPREG(TEST) ;
  DUMPREG(VPOLE) ;
  DUMPREG(IDCODE) ;
  DUMPREG(ADELAY) ;
  DUMPREG(BDELAY) ;
  DUMPREG(ADC) ;
  if ((pPPriv->bt829_id) >= BT827) {
    DUMPREG(VTC) ;
    DUMPREG(CC_STATUS) ;
    DUMPREG(CC_DATA) ;
    DUMPREG(WC_DN) ;
    DUMPREG(P_IO) ; } 
}

/* ------------------------------------------------------------------------ */
/* Private routines */

void bt829_crop(PortPrivPtr pPPriv) 
{
  BTWRITE(CROP,(H(pPPriv->bt829_vdelay)<<6)+(H(pPPriv->bt829_vtotal)<<4)+(H(pPPriv->bt829_hdelay)<<2)+H(pPPriv->overlay_width)) ;
}

void bt829_ctrl(PortPrivPtr pPPriv) 
{
  BTWRITE(CONTROL,((pPPriv->bt829_mux==3)?0xC0:0x00)+
          (pPPriv->bt829_ldec<<5)+(pPPriv->bt829_cbsense<<4)+(H(pPPriv->bt829_luma)<<2)+(H(pPPriv->bt829_sat_u)<<1)+H(pPPriv->bt829_sat_v)) ;
}

void bt829_iform(PortPrivPtr pPPriv) 
{
  BTWRITE(IFORM,(pPPriv->bt829_mux<<5)|(pPPriv->bt829_xtsel<<3)|pPPriv->bt829_format) ;  
}
