#include "Xarch.h"
#include "xaa.h"
#include "xf86.h"
#include "xf86_OSproc.h"
#include "xf86_ansic.h"
#include "xf86Pci.h"
#include "xf86PciInfo.h"
#include "xf86Xinput.h"
#include "xf86fbman.h"
#include "xf86i2c.h"
#include "xf86xv.h"
#include "Xv.h"
#include "xf86Cursor.h"
#include "regionstr.h"
#include "xaalocal.h"

#include "r128.h"
#include "r128_reg.h"
#include "r128_video.h"
#include "r128_i2c.h"
#include "theatre.h"
#include "msp3430.h"
#include "bt829.h"

/*
 * Note:  Only 32-bit MMIO is needed here.
 */



#define CacheByte(___Register) pATI->MMIOCached[CacheSlotOf(___Register) >> 3]
#define CacheBit(___Register)  (0x80U >> (CacheSlotOf(___Register) & 0x07U))

#define SWITCH(a)	if(pPPriv->theatre_num>=0){ RT_##a ; }\
				else\
			if(pPPriv->bt829_addr>0){ bt829_##a ; }


#define XV_ENCODING	"XV_ENCODING"
#define XV_BRIGHTNESS  	"XV_BRIGHTNESS"
#define XV_CONTRAST 	"XV_CONTRAST"
#define XV_SATURATION  	"XV_SATURATION"
#define XV_HUE		"XV_HUE"
#define XV_FREQ		"XV_FREQ"
#define XV_VOLUME	"XV_VOLUME"
#define XV_MUTE		"XV_MUTE"

#define XV_FORMAT		"XV_FORMAT"
#define XV_COLOR_KEY		"XV_COLOR_KEY"
#define XV_COLORKEY		"XV_COLORKEY"
#define XV_PAINT_CLIPLIST	"XV_PAINT_CLIPLIST"

#define MAKE_ATOM(a) MakeAtom(a, sizeof(a) - 1, TRUE)

static Atom xvEncoding, xvBrightness, xvContrast, xvSaturation, xvHue, xv_freq, xv_volume, xv_mute;
static Atom xv_format, xv_color_key, xv_paint_cliplist, xv_colorkey;


#define OVERLAY_TRANSACTION_START { 	if(pATI->CCE2D)R128CCEWaitForIdle(pScrn);   }


/* #define DEBUG */
#define NUM_R128_ATTRIBUTES_PORT0_RT	12

XF86AttributeRec r128_attributes_port0_rt[NUM_R128_ATTRIBUTES_PORT0_RT]={
	{XvSettable,0,-1,XV_FORMAT},
	{XvSettable,0,-1,XV_COLOR_KEY},
	{XvSettable,0,-1,XV_COLORKEY},
	{XvSettable,0,1,XV_PAINT_CLIPLIST},
	{XvSettable,0,9,XV_ENCODING},
	{XvSettable|XvGettable,0,-1,XV_FREQ},
	{XvSettable|XvGettable,-1000,1000,XV_HUE},
	{XvSettable|XvGettable,-1000,1000,XV_SATURATION},
	{XvSettable|XvGettable,-1000,1000,XV_CONTRAST},
	{XvSettable|XvGettable,-1000,1000,XV_BRIGHTNESS},
	{XvSettable|XvGettable,-1000,1000,XV_VOLUME},
	{XvSettable,0,2,XV_MUTE}
	};

#define NUM_R128_ATTRIBUTES_PORT0_BT	11

XF86AttributeRec r128_attributes_port0_bt[NUM_R128_ATTRIBUTES_PORT0_BT]={
	{XvSettable,0,-1,XV_FORMAT},
	{XvSettable,0,-1,XV_COLOR_KEY},
	{XvSettable,0,-1,XV_COLORKEY},
	{XvSettable,0,1,XV_PAINT_CLIPLIST},
	{XvSettable,0,9,XV_ENCODING},
	{XvSettable|XvGettable,0,-1,XV_FREQ},
	{XvSettable|XvGettable,-1000,1000,XV_HUE},
	{XvSettable|XvGettable,-1000,1000,XV_SATURATION},
	{XvSettable|XvGettable,-1000,1000,XV_CONTRAST},
	{XvSettable|XvGettable,-1000,1000,XV_BRIGHTNESS},
	{XvSettable,0,2,XV_MUTE}
	};

#define NUM_R128_ATTRIBUTES_PORT0_IMAGE	9

XF86AttributeRec r128_attributes_port0_image[NUM_R128_ATTRIBUTES_PORT0_IMAGE]={
	{XvSettable,0,-1,XV_FORMAT},
	{XvSettable,0,-1,XV_COLOR_KEY},
	{XvSettable,0,-1,XV_COLORKEY},
	{XvSettable,0,1,XV_PAINT_CLIPLIST},
	{XvSettable,0,9,XV_ENCODING},
	{XvSettable|XvGettable,-1000,1000,XV_HUE},
	{XvSettable|XvGettable,-1000,1000,XV_SATURATION},
	{XvSettable|XvGettable,-1000,1000,XV_CONTRAST},
	{XvSettable|XvGettable,-1000,1000,XV_BRIGHTNESS},
	};




static XF86VideoEncodingRec
InputVideoEncodings[] =
{
    { 0, "pal-composite",		768, 576, { 1, 50 }},
    { 1, "pal-tuner",			768, 576, { 1, 50 }},
    { 2, "pal-svideo",			768, 576, { 1, 50 }},
    { 3, "ntsc-composite",		640, 480, { 1001, 60000 }},
    { 4, "ntsc-tuner",			640, 480, { 1001, 60000 }},
    { 5, "ntsc-svideo",			640, 480, { 1001, 60000 }},
    { 6, "secam-composite",		768, 576, { 1, 50 }},
    { 7, "secam-tuner",			768, 576, { 1, 50 }},
    { 8, "secam-svideo",		768, 576, { 1, 50 }},
    { 9,  "XV_IMAGE",			-1,-1,{-1,-1}},
    { 10, "pal_60-composite",		768, 576, { 1, 50 }},
    { 11, "pal_60-tuner",		768, 576, { 1, 50 }},
    { 12, "pal_60-svideo",		768, 576, { 1, 50 }}
};

static XF86VideoEncodingRec
OutputVideoEncodings[] =
{
    { 0, "pal-composite_adaptor",	704, 576, { 1, 50 }},
    { 1, "pal-svideo",			704, 576, { 1, 50 }},
    { 2, "ntsc-composite_adaptor",	704, 480, { 1001, 60000 }},
    { 3, "ntsc-svideo",			704, 480, { 1001, 60000 }},
};

static XF86VideoFormatRec
InputVideoFormats[] = {
    { 8,  TrueColor }, /* Dithered */
    { 15, TrueColor },
    { 16, TrueColor },
    { 24, TrueColor },
};

static XF86VideoFormatRec
OutputVideoFormats[] = {
    { 8,  TrueColor },
    { 8,  PseudoColor }, /* Using .. */
    { 8,  StaticColor },
    { 8,  GrayScale },
    { 8,  StaticGray }, /* .. TexelLUT */
    { 15, TrueColor },
    { 16, TrueColor },
    { 24, TrueColor },
};

#define NUM_IMAGES_G 3

static XF86ImageRec ImagesG[NUM_IMAGES_G] =
{
   {
	0x32595559,
        XvYUV,
	LSBFirst,
	{'Y','U','Y','2',
	  0x00,0x00,0x00,0x10,0x80,0x00,0x00,0xAA,0x00,0x38,0x9B,0x71},
	16,
	XvPacked,
	1,
	0, 0, 0, 0 ,
	8, 8, 8, 
	1, 2, 2,
	1, 1, 1,
	{'Y','U','Y','V',
	  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
	XvTopToBottom
   },
   {
	0x32315659,
        XvYUV,
	LSBFirst,
	{'Y','V','1','2',
	  0x00,0x00,0x00,0x10,0x80,0x00,0x00,0xAA,0x00,0x38,0x9B,0x71},
	12,
	XvPlanar,
	3,
	0, 0, 0, 0 ,
	8, 8, 8, 
	1, 2, 2,
	1, 2, 2,
	{'Y','V','U',
	  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
	XvTopToBottom
   },
   {
	0x59565955,
        XvYUV,
	LSBFirst,
	{'U','Y','V','Y',
	  0x00,0x00,0x00,0x10,0x80,0x00,0x00,0xAA,0x00,0x38,0x9B,0x71},
	16,
	XvPacked,
	1,
	0, 0, 0, 0 ,
	8, 8, 8, 
	1, 2, 2,
	1, 1, 1,
	{'U','Y','V','Y',
	  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
	XvTopToBottom
   }
};


static Bool
RegionsEqual(RegionPtr A, RegionPtr B);


static void FreeOffscreenBuffers(PortPrivPtr pPPriv)
{
AdaptorPrivPtr pAPriv=pPPriv->pAdaptor;
ScrnInfoPtr pScrn=pAPriv->pScrn;
int buffer;
for(buffer=0;buffer<pPPriv->Buffers;buffer++)
if(pPPriv->pFBArea[buffer]!=NULL){
	xf86FreeOffscreenArea(pPPriv->pFBArea[buffer]);
	pPPriv->pFBArea[buffer]=NULL;
	}
}

static Bool AllocateOffscreenBuffers(PortPrivPtr pPPriv)
{
AdaptorPrivPtr pAPriv=pPPriv->pAdaptor;
ScrnInfoPtr pScrn=pAPriv->pScrn;
long length;
int buffer;
R128InfoPtr	pATI=R128PTR(pScrn);
R128MMIO_VARS();

FreeOffscreenBuffers(pPPriv);
length=(pPPriv->overlay_pixel_size*pPPriv->overlay_width+pPPriv->overlay_pad)*
			pPPriv->overlay_height;
pPPriv->Buffers=2;			
for(buffer=0;buffer<pPPriv->Buffers;buffer++){
	pPPriv->pFBArea[buffer]=xf86AllocateLinearOffscreenArea(pScrn->pScreen,length,1,NULL,NULL,NULL);
	if(pPPriv->pFBArea[buffer]==NULL){
		xf86PurgeUnlockedOffscreenAreas(pScrn->pScreen);
		pPPriv->pFBArea[buffer]=xf86AllocateLinearOffscreenArea(pScrn->pScreen,length,1,NULL,NULL,NULL);
		}
	if(pPPriv->pFBArea[buffer]==NULL){
		xf86DrvMsg(pScrn->scrnIndex,X_ERROR,"Could not allocate buffer (%d bytes)\n",length);
		return FALSE;
		}
	pPPriv->buf_offset[buffer]=(pPPriv->pFBArea[buffer]->box.y1 *pScrn->displayWidth+
			pPPriv->pFBArea[buffer]->box.x1);
	switch(pScrn->depth){
		case 8:
			break;
		case 15:
		case 16:
			pPPriv->buf_offset[buffer]*=2;
			break;
		case 24:
			pPPriv->buf_offset[buffer]*=3;
			break;
		case 32:
			pPPriv->buf_offset[buffer]*=4;
			break;
		default:
		}
	pPPriv->buf_offset[buffer]&=~7;  /* debug */
	/* pPPriv->buf_offset[buffer]=0; */ /* debug */
	xf86DrvMsg(pScrn->scrnIndex,X_INFO,"Allocated buffer=%d offset=0x%08x length=%d\n",buffer,pPPriv->buf_offset[buffer],length);
	}
OVERLAY_TRANSACTION_START
outm(R128_OV0_VID_BUF0_BASE_ADRS,pPPriv->buf_offset[0]);
outm(R128_OV0_VID_BUF1_BASE_ADRS,pPPriv->buf_offset[0]);
outm(R128_OV0_VID_BUF2_BASE_ADRS,pPPriv->buf_offset[0]);

outm(R128_OV0_VID_BUF3_BASE_ADRS,pPPriv->buf_offset[1]);
outm(R128_OV0_VID_BUF4_BASE_ADRS,pPPriv->buf_offset[1]);
outm(R128_OV0_VID_BUF5_BASE_ADRS,pPPriv->buf_offset[1]);
pPPriv->capture_on=FALSE;
pPPriv->first_put_video=TRUE;
return TRUE;
}


struct {
	CARD32  channel_min;
	CARD32	channel_max;
	double	freq_base;
	CARD32	increment;
 	} table[9] =
{   /* min     max       freq  inc */
    {   1,      1,      73.25,  6   },
    {   2,      4,      55.25,  6   },
    {   5,      6,      77.25,  6   },
    {   7,      13,     175.25, 6   },
    {   14,     22,     121.25, 6   },
    {   23,     94,     217.25, 6   },
    {   95,     99,     91.25,  6   },
    {   100,    125,    649.25, 6   },
    {   0,      0,      0,      0   }
};

/*
#define LOW_BAND 91
#define MID_BAND 209
#define MID_HIGH_BAND 497
#define HIGH_BAND 509
*/

#if 0
#define LOW_BAND 0
#define MID_BAND 1782        /* 111.375 MHz */
#define MID_HIGH_BAND 6486
#define HIGH_BAND 14000
#endif


#define LOW_BAND 884           /* 55.25  MHz */
#define MID_BAND 2516          /* 157.25 MHz */
#define MID_HIGH_BAND 12820    /* 801.25 MHz */
#define HIGH_BAND 7220
/*#define HIGH_BAND 12820                /* Specs don't have a mid-high band, only
 a high band! */

void R128_SetTunerChannel (PortPrivPtr pPPriv, CARD32 frequency)
{
    CARD8 retval;
    CARD8 data[16];
    CARD8 counter;
    float Fcar = 45.75;
    CARD16 divider = 0;
    struct{
    	CARD8 div1;
	CARD8 div2;
	CARD8 control;
	CARD8 band;
	} tuner_data;
    I2C_PACKET Packet;
    
switch(pPPriv->encoding){
	case 10:
	case 11:
	case 12:
	case 0:
	case 1:
	case 2: /* PAL/PAL-60 */
	    divider=633+(CARD16)frequency;
		break;
	case 3:
	case 4:
	case 5: /* NTSC */
	    divider=733+(CARD16)frequency;
	    break;
	case 6:
	case 7:
	case 8: /* SECAM */
	    divider=733+(CARD16)frequency;
		break;
	default: /* XV_IMAGE - not used */
		divider=frequency;
	}
	
    tuner_data.div1 = (CARD8)((divider >> 8) & 0xFF);
    tuner_data.div2 = (CARD8)(divider & 0xFF);
    tuner_data.control = 0xCE;
    tuner_data.control = 0x8E;  /* Slower tuning, Not test mode, PLL enabled, 
    				   Normal picture search */


    if ( (frequency >= LOW_BAND) && (frequency < MID_BAND))
    {
        tuner_data.band = 0xA2;
    }
    else if ( (frequency >= MID_BAND) && (frequency < HIGH_BAND))
    {
        tuner_data.band = 0x94;
    }
    else 
    {
        tuner_data.band = 0x34;
    }

    /* Now, set the tuner to the requested channel */
    data[0] = tuner_data.div1;
    data[1] = tuner_data.div2;
    data[2] = tuner_data.control;
    data[3] = tuner_data.band;
    Packet.data = data;
    Packet.start = 1;
    Packet.stop = 1;
    Packet.number_of_bytes = 4;
    Packet.address = pPPriv->tuner_addr;
    counter = 0;
    do
    {
        /*retval = I2C_Set (AddressInfo.Tuner, 4, &data); */
        retval = I2C_Set (pPPriv,&Packet);
        counter++;
    }
    while ((retval != I2C_DONE) && (counter < Packet.number_of_bytes));
} /* R128_SetTunerChannel ()... */


#define TDA9850(a,b)	{\
		data[0]=a;\
		data[1]=b;\
		packet.number_of_bytes=4;\
		packet.address=pPPriv->tda9850;\
		packet.start=1;\
		packet.stop=1;\
		counter=0;\
		do {\
			retval=I2C_Set(pPPriv,&packet);\
			counter++;\
		   } while ((counter<packet.number_of_bytes) && (retval!=I2C_DONE));\
		}

void TDA_setaudio(PortPrivPtr pPPriv)
{
I2C_PACKET packet;
CARD8 data[2];
CARD8 counter;
CARD8 retval;
CARD8 tmp;
AdaptorPrivPtr pAPriv = pPPriv->pAdaptor;
ScrnInfoPtr	pScrn=pAPriv->pScrn;
R128InfoPtr	pATI=R128PTR(pScrn);
R128MMIO_VARS();

packet.data=data;

OVERLAY_TRANSACTION_START

if(pPPriv->tda9850>0){
	if(pPPriv->bt829_mux==2){
	      	TDA9850(0x04,0x0F); TDA9850(0x05,0x0F);
      		TDA9850(0x07,0x07); TDA9850(0x08,0x00);
      		TDA9850(0x09,0x00); TDA9850(0x0A,0x03); 
		} else {
      		TDA9850(0x04,0x07); TDA9850(0x05,0x07); 
		TDA9850(0x06,0x40); TDA9850(0x07,0x07); 
		TDA9850(0x08,0x10); TDA9850(0x09,0x10); 
		TDA9850(0x0A,0x03); 
		}
	
    	TDA9850(0x06,(pPPriv->stereo<<6)|(pPPriv->sap<<7)|(pPPriv->mute?0x8:0)); 
	}
}

void TDA_mute(PortPrivPtr pPPriv)
{
I2C_PACKET packet;
CARD8 data[2];
CARD8 counter;
CARD8 retval;
AdaptorPrivPtr pAPriv = pPPriv->pAdaptor;
ScrnInfoPtr	pScrn=pAPriv->pScrn;
R128InfoPtr	pATI=R128PTR(pScrn);
R128MMIO_VARS();

packet.data=data;
OVERLAY_TRANSACTION_START

if(pPPriv->tda9850>0){
    	TDA9850(0x06,(pPPriv->stereo<<6)|(pPPriv->sap<<7)|(pPPriv->mute?0x8:0x0)); 
	}
}

#define R128EnableBits	((1<<30))

#define R128OverlayOn()        { \
			pPPriv->scale_cntl|=R128EnableBits;\
			outm(R128_OV0_SCALE_CNTL,pPPriv->scale_cntl|pPPriv->video_format);\
			}


#define R128OverlayOff()	{\
			pPPriv->scale_cntl&=~R128EnableBits;\
			outm(R128_OV0_SCALE_CNTL,pPPriv->scale_cntl|pPPriv->video_format);\
			}

/* capture config constants */
#define BUF_TYPE_FIELD		0
#define BUF_TYPE_ALTERNATING	1
#define BUF_TYPE_FRAME		2


#define BUF_MODE_SINGLE		0
#define BUF_MODE_DOUBLE		1
#define BUF_MODE_TRIPLE		2
/* CAP0_CONFIG values */
#define ENABLE_R128_CAPTURE_WEAVE (0x1C000005L | (BUF_MODE_SINGLE <<7) | (BUF_TYPE_FRAME << 4) | ( (pPPriv->theatre_num >=0)?(1L<<23):0) | (1<<29))
#define ENABLE_R128_CAPTURE_BOB (0x1C000005L | (BUF_MODE_SINGLE <<7) | (BUF_TYPE_ALTERNATING << 4) | ( (pPPriv->theatre_num >=0)?(1L<<23):0) | (0<<15) | (1<<29))


void RT_capture_on(PortPrivPtr pPPriv)
{
AdaptorPrivPtr pAPriv = pPPriv->pAdaptor;
ScrnInfoPtr	pScrn=pAPriv->pScrn;
R128InfoPtr	pATI=R128PTR(pScrn);
int v;
CARD32 vclk_ecp_cntl;
CARD32 ecp_div;
R128MMIO_VARS();

if(pPPriv->capture_on)return;
if(pPPriv->overlay_width<1)pPPriv->overlay_width=1;
if(pPPriv->overlay_height<1)pPPriv->overlay_height=1;

xf86DrvMsg(pScrn->scrnIndex,X_INFO,"rage theatre capture_on %dx%d\n",pPPriv->overlay_width,
							pPPriv->overlay_height);

OVERLAY_TRANSACTION_START

		
RT_SetInterlace(pPPriv,1);
RT_SetOutputVideoSize(pPPriv,pPPriv->overlay_width, pPPriv->overlay_height,0,0);
RT_SetTint(pPPriv,pPPriv->iHue);
RT_SetSaturation(pPPriv,pPPriv->iSaturation);
RT_SetBrightness(pPPriv,pPPriv->iBrightness);
RT_SetSharpness(pPPriv,pPPriv->wSharpness);
RT_SetContrast(pPPriv,pPPriv->iContrast);
switch(pPPriv->encoding){
	case 0:
		RT_SetStandard(pPPriv,DEC_PAL | extPAL);
		RT_SetConnector(pPPriv,DEC_COMPOSITE, 0);
		v=24;
		break;
	case 1:
		RT_SetStandard(pPPriv,DEC_PAL | extPAL);
		RT_SetConnector(pPPriv,DEC_TUNER,0);
		v=24;
		break;
	case 2:
		RT_SetStandard(pPPriv,DEC_PAL | extPAL);
		RT_SetConnector(pPPriv,DEC_SVIDEO,0);
		v=24;
		break;
	case 3:
		RT_SetStandard(pPPriv,DEC_NTSC | extNONE);
		RT_SetConnector(pPPriv, DEC_COMPOSITE,0);
		v=23;
		break;
	case 4:
		RT_SetStandard(pPPriv,DEC_NTSC | extNONE);
		RT_SetConnector(pPPriv, DEC_TUNER, 0);
		v=23;
		break;
	case 5:
		RT_SetStandard(pPPriv,DEC_NTSC | extNONE);
		RT_SetConnector(pPPriv, DEC_SVIDEO, 0);
		v=23;
		break;
	case 6:
		RT_SetStandard(pPPriv,DEC_SECAM | extNONE);
		RT_SetConnector(pPPriv, DEC_COMPOSITE, 0);
		v=25;
		break;
	case 7:
		RT_SetStandard(pPPriv,DEC_SECAM | extNONE);
		RT_SetConnector(pPPriv, DEC_TUNER, 0);
		v=25;
		break;
	case 8:
		RT_SetStandard(pPPriv,DEC_SECAM | extNONE);
		RT_SetConnector(pPPriv, DEC_SVIDEO, 0);
		v=25;
		break;
	case 10:
		RT_SetStandard(pPPriv,DEC_PAL | extPAL_60);
		RT_SetConnector(pPPriv,DEC_COMPOSITE, 0);
		v=24;
		break;
	case 11:
		RT_SetStandard(pPPriv,DEC_PAL | extPAL_60);
		RT_SetConnector(pPPriv,DEC_TUNER,0);
		v=24;
		break;
	case 12:
		RT_SetStandard(pPPriv,DEC_PAL | extPAL_60);
		RT_SetConnector(pPPriv,DEC_SVIDEO,0);
		v=24;
		break;
	default:
		return;
	}	

outm(R128_CAP0_BUF0_OFFSET,pPPriv->buf_offset[0]);	
outm(R128_CAP0_BUF0_EVEN_OFFSET,pPPriv->buf_offset[1]);	

outm(R128_CAP0_ONESHOT_BUF_OFFSET,pPPriv->buf_offset[0]);
/*
outm(R128_CAP0_BUF0_OFFSET,0);	
outm(R128_CAP0_BUF0_EVEN_OFFSET,50000);	
*/
outm(R128_CAP0_BUF1_OFFSET,100000);	
outm(R128_CAP0_BUF1_EVEN_OFFSET,150000);	

outm(R128_CAP0_BUF_PITCH,pPPriv->overlay_width*2);
outm(R128_CAP0_H_WINDOW,(2*pPPriv->overlay_width)<<16);
outm(R128_CAP0_V_WINDOW,(((pPPriv->overlay_height>>1)+v-1)<<16)|(v));
outm(R128_OV0_AUTO_FLIP_CNTL,(0<<19)|(0<<18)|(0<<16)|(0<<8)|(0<<6)|(1<<4)|(0)) ;
outm(R128_CAP0_CONFIG,ENABLE_R128_CAPTURE_BOB );
/*outm(R128_VIDEOMUX_CNTL,0x50781); 
outm(R128_VIDEOMUX_CNTL,( inm(R128_VIDEOMUX_CNTL) | 2 | 1 )); */
outm(R128_VIDEOMUX_CNTL,( inm(R128_VIDEOMUX_CNTL) | 1 ));
outm(R128_CAP0_DEBUG,0);
/*outm(R128_CAP0_PORT_MODE_CNTL,(pPPriv->theatre_num>=0)? 2 : 0);   */
outm(R128_CAP0_PORT_MODE_CNTL,(pPPriv->theatre_num>=0)? 1 : 0); 
outm(R128_CAP0_DWNSC_XRATIO,0x10001000);
outm(R128_CAP0_XSHARPNESS,0);
outm(R128_TEST_DEBUG_CNTL, 0);

ecp_div=0; /* hardwired for now.. */
vclk_ecp_cntl=INPLL(pScrn,R128_VCLK_ECP_CNTL);
OUTPLL(R128_VCLK_ECP_CNTL,(vclk_ecp_cntl & 0xFFFFFCFFL)|(ecp_div<<8));

OUTPLL(R128_FCP_CNTL,0x101);
outm(R128_CAP0_TRIG_CNTL,0x11L);


	
pPPriv->capture_on=TRUE;
}

void bt829_capture_on(PortPrivPtr pPPriv)
{
AdaptorPrivPtr pAPriv = pPPriv->pAdaptor;
ScrnInfoPtr	pScrn=pAPriv->pScrn;
R128InfoPtr	pATI=R128PTR(pScrn);
int v;
CARD32 vclk_ecp_cntl;
CARD32 ecp_div;
R128MMIO_VARS();

if(pPPriv->capture_on)return;
if(pPPriv->overlay_width<1)pPPriv->overlay_width=1;
if(pPPriv->overlay_height<1)pPPriv->overlay_height=1;

xf86DrvMsg(pScrn->scrnIndex,X_INFO,"bt829 capture_on %dx%d config=0x%0x\n",pPPriv->overlay_width,
							pPPriv->overlay_height,
							ENABLE_R128_CAPTURE_BOB);
		
switch(pPPriv->encoding){
	case 10:
	case 11:
	case 12:
	case 0:
	case 1:
	case 2: 
		pPPriv->wStandard=DEC_PAL;
		pPPriv->bt829_format=3;
		break;
	case 3:
	case 4:
	case 5:
		pPPriv->wStandard=DEC_NTSC;
		pPPriv->bt829_format=1;
		break;
	case 6:
	case 7:
	case 8:
		pPPriv->wStandard=DEC_SECAM;
		pPPriv->bt829_format=6;
		break;
	}
	
/* pPPriv->scale_cntl=0x61FF0001L | (1<<15);  */
bt829_setformat(pPPriv,pPPriv->bt829_format);
bt829_setcaptsize(pPPriv);

TDA_setaudio(pPPriv);

#if 0
bt829_SetTint(pPPriv,pPPriv->iHue);
bt829_SetSaturation(pPPriv,pPPriv->iSaturation);
bt829_SetBrightness(pPPriv,pPPriv->iBrightness);
bt829_SetContrast(pPPriv,pPPriv->iContrast);
#endif

v=0;
switch(pPPriv->encoding){
	case 0:
		pPPriv->bt829_mux=1;
		v=24;
		break;
	case 1:
		pPPriv->bt829_mux=2;
		v=24;
		break;
	case 2:
		pPPriv->bt829_mux=3;
		v=24;
		break;
	case 3:
		pPPriv->bt829_mux=1;
		v=23;
		break;
	case 4:
		pPPriv->bt829_mux=2;
		v=23;
		break;
	case 5:
		pPPriv->bt829_mux=3;
		v=23;
		break;
	case 6:
		pPPriv->bt829_mux=1;
		v=25;
		break;
	case 7:
		pPPriv->bt829_mux=2;
		v=25;
		break;
	case 8:
		pPPriv->bt829_mux=3;
		v=25;
		break;
	default:
		return;
	}	
bt829_setmux(pPPriv);

OVERLAY_TRANSACTION_START


outm(R128_CAP0_BUF0_OFFSET,pPPriv->buf_offset[0]);	
outm(R128_CAP0_BUF0_EVEN_OFFSET,pPPriv->buf_offset[1]);	

outm(R128_CAP0_ONESHOT_BUF_OFFSET,pPPriv->buf_offset[0]);
/*
outm(R128_CAP0_BUF0_OFFSET,0);	
outm(R128_CAP0_BUF0_EVEN_OFFSET,50000);	
*/

outm(R128_CAP0_BUF1_OFFSET,100000);	
outm(R128_CAP0_BUF1_EVEN_OFFSET,150000);	

outm(R128_CAP0_BUF_PITCH,pPPriv->overlay_width*2);
outm(R128_CAP0_H_WINDOW,(2*pPPriv->overlay_width)<<16);
outm(R128_CAP0_V_WINDOW,(((pPPriv->overlay_height>>1)+v-1)<<16)|(v));
outm(R128_OV0_AUTO_FLIP_CNTL,(0<<19)|(0<<18)|(0<<16)|(0<<8)|(1<<6)|(1<<4)|(0)) ;
inm(R128_OV0_AUTO_FLIP_CNTL);
outm(R128_OV0_AUTO_FLIP_CNTL,(0<<19)|(0<<18)|(0<<16)|(0<<8)|(0<<6)|(1<<4)|(0)) ;
outm(R128_CAP0_CONFIG,ENABLE_R128_CAPTURE_BOB);
/*outm(R128_VIDEOMUX_CNTL,0x50781); 
outm(R128_VIDEOMUX_CNTL,( inm(R128_VIDEOMUX_CNTL) | 2 | 1 )); */
outm(R128_VIDEOMUX_CNTL,( inm(R128_VIDEOMUX_CNTL) | 2 | 1 ));
outm(R128_CAP0_DEBUG,0);
/*outm(R128_CAP0_PORT_MODE_CNTL,(pPPriv->theatre_num>=0)? 2 : 0);   */
outm(R128_CAP0_PORT_MODE_CNTL,0); 
outm(R128_CAP0_DWNSC_XRATIO,0x10001000);
outm(R128_CAP0_XSHARPNESS,0);
outm(R128_TEST_DEBUG_CNTL, 0);

ecp_div=0; /* hardwired for now.. */
vclk_ecp_cntl=INPLL(pScrn,R128_VCLK_ECP_CNTL);
OUTPLL(R128_VCLK_ECP_CNTL,(vclk_ecp_cntl & 0xFFFFFCFFL)|(ecp_div<<8));

OUTPLL(R128_FCP_CNTL,0x101);
outm(R128_CAP0_TRIG_CNTL,0x11L);
	
pPPriv->capture_on=TRUE;

}

void capture_off(PortPrivPtr pPPriv)
{
AdaptorPrivPtr pAPriv = pPPriv->pAdaptor;
ScrnInfoPtr	pScrn=pAPriv->pScrn;
R128InfoPtr	pATI=R128PTR(pScrn);
R128MMIO_VARS();

OVERLAY_TRANSACTION_START

outm(R128_CAP0_TRIG_CNTL,0x0);
OUTPLL(R128_FCP_CNTL,0x404);

}


void setup_buffers(PortPrivPtr pPPriv)
{
int width,height;
AdaptorPrivPtr pAPriv = pPPriv->pAdaptor;
ScrnInfoPtr	pScrn=pAPriv->pScrn;
R128InfoPtr	pATI=R128PTR(pScrn);
R128MMIO_VARS();

switch(pPPriv->wStandard & 0x00FF)
{
	case DEC_NTSC:
		height=480;
		width=640;
		break;
	case DEC_PAL:
		height=576;
		width=768;
		break;
	case DEC_SECAM:
		height=576;
		width=768;
		break;
	default:
		return;
}

if((pPPriv == & pAPriv->Port[0])){

	if((width!=pPPriv->overlay_width)||
	   (height!=pPPriv->overlay_height)||
	   (pPPriv->overlay_pad<0)||
	   (!pPPriv->buffers_allocated)){
		FreeOffscreenBuffers(pPPriv);
		pPPriv->overlay_pad=2;
		pPPriv->overlay_width=width;
		pPPriv->overlay_height=height;
		pPPriv->overlay_pixel_size=2;
		pPPriv->buffers_allocated=AllocateOffscreenBuffers(pPPriv);
		}
	if(!pPPriv->buffers_allocated)return;
	}

}


static int
R128SetPortAttribute(ScrnInfoPtr pScrn,
    Atom attribute, INT32 value, pointer data)
{
PortPrivPtr pPPriv = (PortPrivPtr) data;
AdaptorPrivPtr pAPriv = pPPriv->pAdaptor;
R128InfoPtr	pATI=R128PTR(pScrn);
R128MMIO_VARS();
if((attribute==xv_color_key)||(attribute==xv_colorkey)){
	pPPriv->overlay_key=value;
	outm(R128_OV0_GRAPHICS_KEY_CLR,pPPriv->overlay_key);
	return Success;
	} 
if(attribute==xv_paint_cliplist){
	pPPriv->paint_cliplist=value;
	return Success;
	} 
if(attribute==xvEncoding){
	pPPriv->encoding=value;
	pPPriv->capture_on=FALSE;
	SWITCH(capture_on(pPPriv));
	setup_buffers(pPPriv);
	pPPriv->capture_on=FALSE;
	SWITCH(capture_on(pPPriv));
	pPPriv->first_put_video=TRUE;
	/* reinitialize MSP3430 */
	EnableMSP3430(pPPriv);
	return Success;
	}
if(attribute==xv_freq){
	R128_SetTunerChannel(pPPriv, value);
	return Success;
	}
if(attribute==xv_volume){
	if(value>1000)value=1000;
	if(value<-1000)value=-1000;
	if(value==pPPriv->volume)return Success; /* don't make extra i2c transactions */
	
	pPPriv->volume=value;
	if(!pPPriv->mute)MSP3430_SetVolume(pPPriv,0x40+ (((pPPriv->volume+1000)*0x3F)/2000));
	return Success;
	}
if(attribute==xv_mute){
	if(value==pPPriv->mute)return Success; /* don't make extra i2c transactions */	
	pPPriv->mute=value;
	if(value){
		if(pAPriv->Port[0].msp3430_addr>0)MSP3430_SetVolume(pPPriv, 0);
		TDA_mute(pPPriv);
		} else {
		if(pAPriv->Port[0].msp3430_addr>0)MSP3430_SetVolume(pPPriv, 0x40+ (((pPPriv->volume+1000)*0x3F)/2000));
		TDA_mute(pPPriv);
		}
	return Success;
	}
if(attribute==xvSaturation){
	pPPriv->iSaturation=value;
	SWITCH(SetSaturation(pPPriv,pPPriv->iSaturation));
	return Success;
	}
if(attribute==xvBrightness){
	pPPriv->iBrightness=value;
	SWITCH(SetBrightness(pPPriv,pPPriv->iBrightness));
	return Success;
	}
if(attribute==xvHue){
	pPPriv->iHue=value;
	SWITCH(SetTint(pPPriv,pPPriv->iHue));
	return Success;
	}
if(attribute==xvContrast){
	pPPriv->iContrast=value;
	SWITCH(SetContrast(pPPriv,pPPriv->iContrast));
	return Success;
	}
xf86DrvMsgVerb(pScrn->scrnIndex,X_INFO,1,"R128SetPortAttribute unknown\n");
return BadMatch;
}

static int
R128GetPortAttribute(ScrnInfoPtr pScrn, 
    Atom attribute, INT32 *value, pointer data)
{
PortPrivPtr pPPriv = (PortPrivPtr) data;
AdaptorPrivPtr pAPriv = pPPriv->pAdaptor;
R128InfoPtr	pATI=R128PTR(pScrn);
R128MMIO_VARS();
if(attribute==xv_freq){
	*value=0;
	return Success;
	}
if(attribute==xv_volume){
	*value=pPPriv->volume;
	return Success;
	}
if(attribute==xv_mute){
	*value=pPPriv->mute;
	return Success;
	}
if(attribute==xvHue){
	*value=pPPriv->iHue;
	return Success;
	}
if(attribute==xvSaturation){
	*value=pPPriv->iSaturation;
	return Success;
	}
if(attribute==xvContrast){
	*value=pPPriv->iContrast;
	return Success;
	}
if(attribute==xvBrightness){
	*value=pPPriv->iBrightness;
	return Success;
	}

xf86DrvMsgVerb(pScrn->scrnIndex,X_INFO,1,"R128GetPortAttribute unknown\n");

return BadMatch;
}



static int
R128PutVideo(ScrnInfoPtr pScrn,
    short src_x, short src_y, short drw_x, short drw_y,
    short src_w, short src_h, short drw_w, short drw_h,
    RegionPtr clipBoxes, pointer data)
{
PortPrivPtr pPPriv = (PortPrivPtr) data;
AdaptorPrivPtr pAPriv = pPPriv->pAdaptor;
R128InfoPtr	pATI=R128PTR(pScrn);
R128MMIO_VARS();
int i;
short width,height;
int id;
int srcPitch,srcPitch2;
int dstPitch;
int pitch,new_h,offset,offset2,offset3;
CARD32 tmp;
int top,left,npixels,nlines,step_by;
INT32 x1,x2,y1,y2;
INT32 d_x,d_y,d_width,d_height;
CARD32 video_format;
BoxPtr pbox;

xf86DrvMsg(pScrn->scrnIndex,X_INFO,"R128PutVideo %dx%d+%d+%d %p\n",src_w,src_h,src_x,src_y,clipBoxes);

/* we want drw_x and _y be actual coordinates of the window,
   not coordinates in the fb
   
   note that filling with base color is done using clipBoxes
   and thus is independendent of drw_x, drw_y */

drw_x-=pScrn->frameX0;
drw_y-=pScrn->frameY0;

OVERLAY_TRANSACTION_START

id=0x59565955;

video_format=(pPPriv->theatre_num>=0) ? (0xb<<8) : (0xb<<8);
if(video_format!=pPPriv->video_format){
	xf86DrvMsg(pScrn->scrnIndex,X_INFO,"R128PutVideo new format: id=0x%08x format=0x%08x\n",id,video_format);
	}

switch(pPPriv->wStandard & 0x00FF)
{
	case DEC_NTSC:
		height=480;
		width=640;
		break;
	case DEC_PAL:
		height=576;
		width=768;
		break;
	case DEC_SECAM:
		height=576;
		width=768;
		break;
	default:
		return BadMatch;
}

/* for some reason the window background gets repainted 
    after first PutVideo call when using xawtv.. So I changed
    PutVideo to always repaint the background (unlike PutImage)
    Presumably there should _not_ be too many PutVideo calls */
pPPriv->first_put_video=TRUE; 

if((pPPriv == &( pAPriv->Port[0]))){
	setup_buffers(pPPriv);
	if(!pPPriv->buffers_allocated)return BadMatch;
	/* clip output rectangle */
	d_x=drw_x;
	d_y=drw_y;
	d_width=drw_w;
	d_height=drw_h;
	if(drw_x<0){
		drw_w+=drw_x;
		drw_x=0;
		}
	if(drw_y<0){
		drw_h+=drw_y;
		drw_y=0;
		}
	if(drw_x+drw_w>pScrn->pScreen->width){
		drw_w=pScrn->pScreen->width-drw_x;
		}
	if(drw_y+drw_h>pScrn->pScreen->height){
		drw_h=pScrn->pScreen->height-drw_y;
		}
	if((drw_w<=0)||(drw_h<=0)){
		/* this should not happen,
			since we are outside of visible screen,
			 but just in case */
		return Success;
		}
		
   	x1 = src_x;
   	x2 = src_x + src_w;
   	y1 = src_y;
   	y2 = src_y + src_h;

	dstPitch=pPPriv->overlay_width*pPPriv->overlay_pixel_size;
	srcPitch=width;
    	/* copy data */
    	top = y1 >> 16;
    	left = (x1 >> 16) & ~1;
    	npixels = ((((x2 + 0xffff) >> 16) + 1) & ~1) - left;
    	left <<= 1;
	switch(id) {
	   	case 0x32315659:
			srcPitch = (width + 3) & ~3;
			offset2 = srcPitch * height;
			srcPitch2 = ((width >> 1) + 3) & ~3;
			offset3 = (srcPitch2 * (height >> 1)) + offset2;
			nlines = ((((y2 + 0xffff) >> 16) + 1) & ~1) - top;			
			break;
	   	case 0x59565955:
	   	case 0x32595559:
	   	default:
			/*buf += (top * srcPitch) + left;*/
			nlines = ((y2 + 0xffff) >> 16) - top;
			srcPitch = (width << 1);
    			break;
   		} 
	nlines=src_h; /* ?*/
	npixels=src_w;

	if(npixels>pPPriv->overlay_width)npixels=pPPriv->overlay_width;
	if(nlines>pPPriv->overlay_height)nlines=pPPriv->overlay_height;

	/* adjust source rectangle */	
	src_x+=((drw_x-d_x)*src_w)/d_width;
	src_y+=((drw_y-d_y)*src_h)/d_height;
		
	src_w=(src_w * drw_w)/d_width;
	src_h=(src_h * drw_h)/d_height;

	/* calculate step_by factor */
	step_by=src_w/(drw_w*2);
	switch(step_by){
		case 0:
			outm(R128_OV0_STEP_BY,0x101);
			step_by=1;
			break;
		case 1:
			outm(R128_OV0_STEP_BY,0x202);
			step_by=2;
			break;
		case 2:
		case 3:
			outm(R128_OV0_STEP_BY,0x303);
			step_by=4;
			break;
		default:
			outm(R128_OV0_STEP_BY,0x404);
			step_by=8;
			break;
		}
	

	offset=(src_x+src_y*width)*pPPriv->overlay_pixel_size; /* in bytes */
	
	
	pPPriv->vert_inc=(src_h<<12)/(drw_h);
	pPPriv->horz_inc=(src_w<<12)/(drw_w*step_by);

	pPPriv->drw_w=drw_w;
	pPPriv->drw_h=drw_h;

	#if 0	
	xf86DrvMsg(pScrn->scrnIndex,X_INFO,"%dx%d  %d\n",drw_w,drw_h,offset);
	if(!(inm(R128_CRTC_STATUS)&2)){
	#if 0
		xf86DrvMsg(pScrn->scrnIndex,X_INFO,"too fast");
	#endif	
		/* vlank has not happened */
		return Success; /* we are too fast */
		}
	#endif
	OVERLAY_TRANSACTION_START

	outm(R128_OV0_Y_X_START,((drw_x))|(drw_y<<16)|(1<<31));
	outm(R128_OV0_Y_X_END,((drw_x+drw_w))|((drw_y+drw_h)<<16));
	outm(R128_OV0_H_INC,(pPPriv->horz_inc)|((pPPriv->horz_inc<<15)));
	outm(R128_OV0_V_INC,(pPPriv->vert_inc<<8));
	outm(R128_OV0_P1_BLANK_LINES_AT_TOP,0xfff|((src_h-1)<<16));
	outm(R128_OV0_P23_BLANK_LINES_AT_TOP,0xfff|((src_h-1)<<16));
	outm(R128_OV0_VID_BUF_PITCH0_VALUE,pPPriv->overlay_width<<1);
	outm(R128_OV0_VID_BUF_PITCH1_VALUE,pPPriv->overlay_width<<1);
	outm(R128_OV0_P1_X_START_END,(src_w-1)|((src_x&0xf)<<16));
	outm(R128_OV0_P2_X_START_END,(src_w-1)|((src_x&0xf)<<16));
	outm(R128_OV0_P3_X_START_END,(src_w-1)|((src_x&0xf)<<16));

	outm(R128_OV0_VID_BUF0_BASE_ADRS,(pPPriv->buf_offset[0]+offset)&(~0xf));
	outm(R128_OV0_VID_BUF1_BASE_ADRS,(pPPriv->buf_offset[0]+offset)&(~0xf));
	outm(R128_OV0_VID_BUF2_BASE_ADRS,(pPPriv->buf_offset[0]+offset)&(~0xf));

	outm(R128_OV0_VID_BUF3_BASE_ADRS,(pPPriv->buf_offset[1]+offset)&(~0xf));
	outm(R128_OV0_VID_BUF4_BASE_ADRS,(pPPriv->buf_offset[1]+offset)&(~0xf));
	outm(R128_OV0_VID_BUF5_BASE_ADRS,(pPPriv->buf_offset[1]+offset)&(~0xf));
	
	if(pPPriv->paint_cliplist && (!RegionsEqual(&pPPriv->prev_cliplist,clipBoxes) || pPPriv->first_put_video)){
		pPPriv->first_put_video=FALSE;
		REGION_COPY(pScreen,&pPPriv->prev_cliplist,clipBoxes);
		/* XAAFillSolidRects(pScrn,pPPriv->overlay_key,GXcopy, ~0, 
			REGION_NUM_RECTS(clipBoxes),
			REGION_RECTS(clipBoxes));  */
		pATI->accel->SetupForSolidFill(pScrn,pPPriv->overlay_key,GXcopy,~0);
		/* pATI->accel->SetupForSolidFill(pScrn,0xffffff,GXcopy,~0); */
		pbox=REGION_RECTS(clipBoxes);
		for(i=REGION_NUM_RECTS(clipBoxes);i;i--,pbox++){
			pATI->accel->SubsequentSolidFillRect(pScrn,pbox->x1,pbox->y1,
								pbox->x2-pbox->x1,pbox->y2-pbox->y1);
			} /* I have no idea why XAAFillSolidRects fails */
		}
	
	/* setup capture */
	
	SWITCH(capture_on(pPPriv));
	
	/*****************/
	
	#if 0
	if(pPPriv->auto_flip_cntl&(1<<6)){
		pPPriv->auto_flip_cntl&=~(1<<6);
		} else 
		{
		pPPriv->auto_flip_cntl|=(1<<6);
		}

		
	outm(R128_OV0_AUTO_FLIP_CNTL,pPPriv->auto_flip_cntl | (pPPriv->current_buffer*3) );
	outm(R128_CRTC_STATUS,inm(R128_CRTC_STATUS)&(~2)); /* clear save vblank flag */
	/*
	pPPriv->current_buffer=!pPPriv->current_buffer;
	*/
	#endif
	
	outm(R128_OV0_DEINTERLACE_PATTERN,0x0);

	R128OverlayOn();
	
	
	return Success;
	}

return BadValue;
}

static int
R128PutStill(ScrnInfoPtr pScrn,
    short vid_x, short vid_y, short drw_x, short drw_y,
    short vid_w, short vid_h, short drw_w, short drw_h,
    RegionPtr clipBoxes, pointer data)
{
xf86DrvMsgVerb(pScrn->scrnIndex,X_INFO,1,"R128PutStill\n");

return Success;
}

static int
R128GetVideo(ScrnInfoPtr pScrn,
    short vid_x, short vid_y, short drw_x, short drw_y,
    short vid_w, short vid_h, short drw_w, short drw_h,
    RegionPtr clipBoxes, pointer data)
{
xf86DrvMsgVerb(pScrn->scrnIndex,X_INFO,1,"R128GetVideo\n");

return BadValue;
}

static int
R128GetStill(ScrnInfoPtr pScrn,
    short vid_x, short vid_y, short drw_x, short drw_y,
    short vid_w, short vid_h, short drw_w, short drw_h,
    RegionPtr clipBoxes, pointer data)
{
xf86DrvMsgVerb(pScrn->scrnIndex,X_INFO,1,"R128GetStill\n");
return BadMatch;
}

static void
R128StopVideo(ScrnInfoPtr pScrn, pointer data, Bool exit)
{
PortPrivPtr pPPriv = (PortPrivPtr) data;
AdaptorPrivPtr pAPriv = pPPriv->pAdaptor;
R128InfoPtr	pATI=R128PTR(pScrn);
R128MMIO_VARS();
xf86DrvMsgVerb(pScrn->scrnIndex,X_INFO,1,"R128StopVideo\n");

OVERLAY_TRANSACTION_START

R128OverlayOff();
if(exit){
	capture_off(pPPriv);
	R128SetPortAttribute(pScrn,xv_mute,1,data);
	FreeOffscreenBuffers(pPPriv);
	pPPriv->mute=1;
	TDA_mute(pPPriv);
	MSP3430_SetVolume(pPPriv,0);
	}
}

static void
R128QueryBestSize(ScrnInfoPtr pScrn, Bool motion,
    short vid_w, short vid_h, short drw_w, short drw_h,
    unsigned int *p_w, unsigned int *p_h, pointer data)
{
*p_w=drw_w;
*p_h=drw_h;
}

static void
R128CopyMungedData(
   unsigned char *src1,
   unsigned char *src2,
   unsigned char *src3,
   unsigned char *dst1,
   int srcPitch,
   int srcPitch2,
   int dstPitch,
   int h,
   int w
){
   CARD32 *dst = (CARD32*)dst1;
   int i, j;

   dstPitch >>= 2;
   w >>= 1;

   for(j = 0; j < h; j++) {
	for(i = 0; i < w; i++) {
	    dst[i] = src1[i << 1] | (src1[(i << 1) + 1] << 16) |
		     (src3[i] << 8) | (src2[i] << 24);
	}
	dst += dstPitch;
	src1 += srcPitch;
	if(j & 1) {
	    src2 += srcPitch2;
	    src3 += srcPitch2;
	}
   }
}


static void R128CopyData(
  unsigned char *src,
  unsigned char *dst,
  int srcPitch,
  int dstPitch,
  int h,
  int w
){
    w <<= 1;
    while(h--) {
	memcpy(dst, src, w);
	src += srcPitch;
	dst += dstPitch;
    }
}


static Bool
RegionsEqual(RegionPtr A, RegionPtr B)
{
    int *dataA, *dataB;
    int num;

    num = REGION_NUM_RECTS(A);
    if(num != REGION_NUM_RECTS(B))
	return FALSE;

    if((A->extents.x1 != B->extents.x1) ||
       (A->extents.x2 != B->extents.x2) ||
       (A->extents.y1 != B->extents.y1) ||
       (A->extents.y2 != B->extents.y2))
	return FALSE;

    dataA = (int*)REGION_RECTS(A);
    dataB = (int*)REGION_RECTS(B);

    while(num--) {
	if((dataA[0] != dataB[0]) || (dataA[1] != dataB[1]))
	   return FALSE;
	dataA += 2; 
	dataB += 2;
    }

    return TRUE;
}



int R128PutImage( ScrnInfoPtr pScrn, 
	short src_x, short src_y, short drw_x, short drw_y,
	short src_w, short src_h, short drw_w, short drw_h,
	int id, unsigned char* buf, short width, short height, Bool sync,
	RegionPtr clipBoxes, pointer data )
{
PortPrivPtr pPPriv = (PortPrivPtr) data;
AdaptorPrivPtr pAPriv = pPPriv->pAdaptor;
R128InfoPtr	pATI=R128PTR(pScrn);
int i;
int srcPitch,srcPitch2;
int dstPitch;
int pitch,new_h,offset,offset2,offset3;
CARD32 tmp;
int top,left,npixels,nlines,step_by;
INT32 x1,x2,y1,y2;
INT32 d_x,d_y,d_width,d_height;
CARD32 video_format;
R128MMIO_VARS();
BoxPtr pbox;

/* we want drw_x and _y be actual coordinates of the window,
   not coordinates in the fb
   
   note that filling with base color is done using clipBoxes
   and thus is independendent of drw_x, drw_y */

drw_x-=pScrn->frameX0;
drw_y-=pScrn->frameY0;


/*xf86DrvMsg(pScrn->scrnIndex,X_INFO,"R128PutImage drw_x=%d drw_w=%d width=%d height=%d sync=%d\n",drw_x,drw_w,width,height,sync);
*/
switch(id) {
   	case 0x32315659: /* 2YUV */
		video_format=0xB<<8;
		break;
	case 0x59565955:  /* YVYU */
		video_format=0xB<<8;
		break;
   	case 0x32595559:  /* 2YUY */
		video_format=0xC<<8;
		break;
   	default:
		return BadValue;
  		
   	} 
if(video_format!=pPPriv->video_format){
	xf86DrvMsg(pScrn->scrnIndex,X_INFO,"R128PutImage new format: id=0x%08x format=0x%08x\n",id,video_format);
	}

OVERLAY_TRANSACTION_START

if(pPPriv->capture_on){
	capture_off(pPPriv);
	}

if((pPPriv == & pAPriv->Port[0])){
	if((width!=pPPriv->overlay_width)||
	   (height!=pPPriv->overlay_height)||
	   (id!=pPPriv->overlay_id)||
	   (video_format!=pPPriv->video_format)||
	   (pPPriv->overlay_pad<0)||
	   (!pPPriv->buffers_allocated)){
		FreeOffscreenBuffers(pPPriv);
		pPPriv->overlay_pad=2;
		pPPriv->overlay_width=width;
		pPPriv->overlay_height=height;
		pPPriv->overlay_pixel_size=2;
		pPPriv->overlay_id=id;
		pPPriv->buffers_allocated=AllocateOffscreenBuffers(pPPriv);
		}
	if(!pPPriv->buffers_allocated)return BadMatch;
	/* clip output rectangle */
	d_x=drw_x;
	d_y=drw_y;
	d_width=drw_w;
	d_height=drw_h;
	if(drw_x<0){
		drw_w+=drw_x;
		drw_x=0;
		}
	if(drw_y<0){
		drw_h+=drw_y;
		drw_y=0;
		}
	if(drw_x+drw_w>pScrn->pScreen->width){
		drw_w=pScrn->pScreen->width-drw_x;
		}
	if(drw_y+drw_h>pScrn->pScreen->height){
		drw_h=pScrn->pScreen->height-drw_y;
		}
	if((drw_w<=0)||(drw_h<=0)){
		/* this should not happen,
			since we are outside of visible screen,
			 but just in case */
		return Success;
		}
		
   	x1 = src_x;
   	x2 = src_x + src_w;
   	y1 = src_y;
   	y2 = src_y + src_h;

	dstPitch=pPPriv->overlay_width*pPPriv->overlay_pixel_size;
	srcPitch=width;
    	/* copy data */
    	top = y1 >> 16;
    	left = (x1 >> 16) & ~1;
    	npixels = ((((x2 + 0xffff) >> 16) + 1) & ~1) - left;
    	left <<= 1;
	switch(id) {
	   	case 0x32315659:
			srcPitch = (width + 3) & ~3;
			offset2 = srcPitch * height;
			srcPitch2 = ((width >> 1) + 3) & ~3;
			offset3 = (srcPitch2 * (height >> 1)) + offset2;
			nlines = ((((y2 + 0xffff) >> 16) + 1) & ~1) - top;			
			break;
	   	case 0x59565955:
	   	case 0x32595559:
	   	default:
			buf += (top * srcPitch) + left;
			nlines = ((y2 + 0xffff) >> 16) - top;
			srcPitch = (width << 1);
    			break;
   		} 
	nlines=src_h; /* ?*/
	npixels=src_w;

	if(npixels>pPPriv->overlay_width)npixels=pPPriv->overlay_width;
	if(nlines>pPPriv->overlay_height)nlines=pPPriv->overlay_height;

	/* adjust source rectangle */	
	src_x+=((drw_x-d_x)*src_w)/d_width;
	src_y+=((drw_y-d_y)*src_h)/d_height;
		
	src_w=(src_w * drw_w)/d_width;
	src_h=(src_h * drw_h)/d_height;

	/* calculate step_by factor */
	step_by=src_w/(drw_w*2);
	switch(step_by){
		case 0:
			outm(R128_OV0_STEP_BY,0x101);
			step_by=1;
			break;
		case 1:
			outm(R128_OV0_STEP_BY,0x202);
			step_by=2;
			break;
		case 2:
		case 3:
			outm(R128_OV0_STEP_BY,0x303);
			step_by=4;
			break;
		default:
			outm(R128_OV0_STEP_BY,0x404);
			step_by=8;
			break;
		}
	

	offset=(src_x+src_y*width)*pPPriv->overlay_pixel_size; /* in bytes */
	
	
	pPPriv->vert_inc=(src_h<<12)/(drw_h);
	pPPriv->horz_inc=(src_w<<12)/(drw_w*step_by);


	pPPriv->drw_w=drw_w;
	pPPriv->drw_h=drw_h;
		
	if(!(inm(R128_CRTC_STATUS)&2)){
	#if 0
		xf86DrvMsg(pScrn->scrnIndex,X_INFO,"too fast");
	#endif	
		/* vlank has not happened */
		return Success; /* we are too fast */
		}

	outm(R128_OV0_Y_X_START,((drw_x))|(drw_y<<16)|(1<<31));
	outm(R128_OV0_Y_X_END,((drw_x+drw_w))|((drw_y+drw_h)<<16));
	outm(R128_OV0_H_INC,(pPPriv->horz_inc)|((pPPriv->horz_inc<<15)));
	outm(R128_OV0_V_INC,(pPPriv->vert_inc<<8));
	outm(R128_OV0_P1_BLANK_LINES_AT_TOP,0xfff|((src_h-1)<<16));
	outm(R128_OV0_P23_BLANK_LINES_AT_TOP,0xfff|((src_h-1)<<16));
	outm(R128_OV0_VID_BUF_PITCH0_VALUE,pPPriv->overlay_width<<1);
	outm(R128_OV0_VID_BUF_PITCH1_VALUE,pPPriv->overlay_width<<1);
	outm(R128_OV0_P1_X_START_END,(src_w-1)|((src_x&0xf)<<16));
	outm(R128_OV0_P2_X_START_END,(src_w-1)|((src_x&0xf)<<16));
	outm(R128_OV0_P3_X_START_END,(src_w-1)|((src_x&0xf)<<16));

	outm(R128_OV0_VID_BUF0_BASE_ADRS,(pPPriv->buf_offset[0]+offset)&(~0xf));
	outm(R128_OV0_VID_BUF1_BASE_ADRS,(pPPriv->buf_offset[0]+offset)&(~0xf));
	outm(R128_OV0_VID_BUF2_BASE_ADRS,(pPPriv->buf_offset[0]+offset)&(~0xf));

	outm(R128_OV0_VID_BUF3_BASE_ADRS,(pPPriv->buf_offset[1]+offset)&(~0xf));
	outm(R128_OV0_VID_BUF4_BASE_ADRS,(pPPriv->buf_offset[1]+offset)&(~0xf));
	outm(R128_OV0_VID_BUF5_BASE_ADRS,(pPPriv->buf_offset[1]+offset)&(~0xf));
	
	if(pPPriv->paint_cliplist && !RegionsEqual(&pPPriv->prev_cliplist,clipBoxes)){
		REGION_COPY(pScreen,&pPPriv->prev_cliplist,clipBoxes);
		/* XAAFillSolidRects(pScrn,pPPriv->overlay_key,GXcopy, ~0, 
			REGION_NUM_RECTS(clipBoxes),
			REGION_RECTS(clipBoxes)); */
		pATI->accel->SetupForSolidFill(pScrn,pPPriv->overlay_key,GXcopy,~0);
		pbox=REGION_RECTS(clipBoxes);
		for(i=REGION_NUM_RECTS(clipBoxes);i;i--,pbox++){
			pATI->accel->SubsequentSolidFillRect(pScrn,pbox->x1,pbox->y1,
								pbox->x2-pbox->x1,pbox->y2-pbox->y1);
			} /* I have no idea why XAAFillSolidRects fails */
		}

	switch(id) {
	   	case 0x32315659:
			R128CopyMungedData(buf + (top * srcPitch) + (left >> 1), 
				  buf + offset2, buf + offset3, pATI->FB+pPPriv->buf_offset[pPPriv->current_buffer],
				  srcPitch, srcPitch2, dstPitch, nlines, npixels);
			break;
		case 0x59565955:
	   	case 0x32595559:
	   	default:
			R128CopyData(buf,pATI->FB+pPPriv->buf_offset[pPPriv->current_buffer],srcPitch,dstPitch,nlines,npixels);
  		
   	} 
	if(pPPriv->auto_flip_cntl&(1<<6)){
		pPPriv->auto_flip_cntl&=~(1<<6);
		} else 
		{
		pPPriv->auto_flip_cntl|=(1<<6);
		}

		
	outm(R128_OV0_AUTO_FLIP_CNTL,pPPriv->auto_flip_cntl | (pPPriv->current_buffer*3) );
	outm(R128_CRTC_STATUS,inm(R128_CRTC_STATUS)&(~2)); /* clear save vblank flag */
	pPPriv->current_buffer=!pPPriv->current_buffer;
	outm(R128_OV0_DEINTERLACE_PATTERN,0xAAAAA);
	
	R128OverlayOn();
	
	return Success;
	}
return BadMatch;
}

int R128ReputImage( ScrnInfoPtr pScrn, short drw_x, short drw_y,
        RegionPtr clipBoxes, pointer data )
{
PortPrivPtr pPPriv = (PortPrivPtr) data;
AdaptorPrivPtr pAPriv = pPPriv->pAdaptor;
R128InfoPtr	pATI=R128PTR(pScrn);
R128MMIO_VARS();

OVERLAY_TRANSACTION_START

if(pPPriv == & pAPriv->Port[0]){
	if(drw_x<0)drw_x=0;
	if(drw_y<0)drw_y=0;
	outm(R128_OV0_Y_X_START,((drw_x))|((drw_y)<<16)|(1<<31));
	outm(R128_OV0_Y_X_END,((drw_x+pPPriv->drw_w))|((drw_y+pPPriv->drw_h)<<16));
	R128OverlayOn();
	return Success;
	}
return BadMatch;
}

static int 
R128QueryImageAttributesG(ScrnInfoPtr pScrn, 
  int id, 
  unsigned short *w, unsigned short *h, 
  int *pitches, int *offsets)
{
    int size, tmp;

    if(*w > 1024) *w = 1024;
    if(*h > 1024) *h = 1024;

    *w = (*w + 1) & ~1;
    if(offsets) offsets[0] = 0;

    switch(id) {
    case 0x32315659:
	*h = (*h + 1) & ~1;
	size = (*w + 3) & ~3;
	if(pitches) pitches[0] = size;
	size *= *h;
	if(offsets) offsets[1] = size;
	tmp = ((*w >> 1) + 3) & ~3;
	if(pitches) pitches[1] = pitches[2] = tmp;
	tmp *= (*h >> 1);
	size += tmp;
	if(offsets) offsets[2] = size;
	size += tmp;
	break;
    case 0x59565955:
    case 0x32595559:
    default:
	size = *w << 1;
	if(pitches) pitches[0] = size;
	size *= *h;
	break;
    }

    return size;
}


static void
AdaptorPrivUninit(AdaptorPrivPtr pAPriv)
{
/*
if(!pAPriv->atip){
	xf86DestroyI2CDev(&pAPriv->Port[0].I2CDev,FALSE);
	}
*/
xf86DrvMsgVerb(pAPriv->pScrn->scrnIndex,X_INFO,1,"ati_video: AdaptorPrivUninit\n");

xfree(pAPriv);
}

#if 0
static int r128_pollscreen(ScrnInfoPtr pScrn,AdaptorPrivPtr pAPriv,int sig)
{
R128InfoPtr pATI=R128PTR(pScrn);
CARD8 save,pll_vclk_cntl, pll_ref_div, vclk_post_div, vclk_fb_div,ecp_div;
int i;
double dotclock,refclock;
#define port   (pAPriv->Port[0])

if(sig  && (inm(pATI->CPIO_CRTC_OFF_PITCH)==port.save_off_pitch) 
	&& (inm(pATI->CPIO_CRTC_H_TOTAL_DISP==port.save_h_total_disp))
	&& (inm(pATI->CPIO_CRTC_V_TOTAL_DISP==port.save_v_total_disp))) return 0;

port.save_off_pitch=inm(pATI->CPIO_CRTC_OFF_PITCH);
port.save_h_total_disp=inm(pATI->CPIO_CRTC_H_TOTAL_DISP);
port.save_v_total_disp=inm(pATI->CPIO_CRTC_V_TOTAL_DISP);

save=inb(pATI->CPIO_CLOCK_CNTL+1);
i=inb(pATI->CPIO_CLOCK_CNTL) & 3;

outb(pATI->CPIO_CLOCK_CNTL+1,2<<2); pll_ref_div=inb(pATI->CPIO_CLOCK_CNTL+2);
outb(pATI->CPIO_CLOCK_CNTL+1,5<<2); pll_vclk_cntl=inb(pATI->CPIO_CLOCK_CNTL+2);
outb(pATI->CPIO_CLOCK_CNTL+1,6<<2); vclk_post_div=(inb(pATI->CPIO_CLOCK_CNTL+2)>>(i+i))&3;
outb(pATI->CPIO_CLOCK_CNTL+1,(7+i)<<2); vclk_fb_div=inb(pATI->CPIO_CLOCK_CNTL+2);
refclock=(pATI->ReferenceNumerator*1.0)/(157500.0/14.31818);
dotclock=2.0*refclock*vclk_fb_div / (pll_ref_div*(1<<vclk_post_div));
ecp_div=dotclock/(125.0);
xf86DrvMsg(pScrn->scrnIndex,X_INFO,"m64_pollscreen dotclock=%g ecp_div=%d\n",dotclock,ecp_div);
if(ecp_div>2)ecp_div=2;
if((pll_vclk_cntl & 0x03)!=0x03)ecp_div=0;
if((pll_vclk_cntl & 0x30)!=ecp_div<<4){
	outb(pATI->CPIO_CLOCK_CNTL+1,(5<<2)|2);
	outb(pATI->CPIO_CLOCK_CNTL+2,(pll_vclk_cntl & 0xCF) | (ecp_div<<4));
	}
	
outb(pATI->CPIO_CLOCK_CNTL+1,save);
#undef port
}
#endif
static AdaptorPrivPtr
AdaptorPrivInit(ScrnInfoPtr pScrn)
{
R128InfoPtr	pATI=R128PTR(pScrn);
AdaptorPrivPtr pAPriv;
R128MMIO_VARS();

pAPriv=(AdaptorPrivPtr) xcalloc(2,sizeof(AdaptorPrivRec));
xf86DrvMsg(pScrn->scrnIndex,X_INFO,"%s: AdaptorPrivInit\n",__FILE__);
if(pAPriv){
	pAPriv->pScrn=pScrn;
	pAPriv->Port[0].pAdaptor=pAPriv;
	pAPriv->Port[0].overlay_pad=-1;
	pAPriv->Port[0].scale_cntl=(1<<29)|(1<<24)|(1<<15)|1;
	pAPriv->Port[0].video_format=0xBL<<8;
	pAPriv->Port[0].overlay_key=0x010203L;
	pAPriv->Port[0].overlay_id=0;
	pAPriv->Port[0].overlay_width=0;
	pAPriv->Port[0].overlay_height=0;
	pAPriv->Port[0].current_buffer=1;
	pAPriv->Port[0].auto_flip_cntl=(2<<8);
	pAPriv->Port[0].paint_cliplist=1;
	pAPriv->Port[0].buffers_allocated=FALSE;
	pAPriv->Port[0].capture_on=FALSE;
	pAPriv->Port[0].iHue=0;
	pAPriv->Port[0].iSaturation=0;
	pAPriv->Port[0].wSharpness=RT_NORM_SHARPNESS;
	pAPriv->Port[0].iContrast=0;
	pAPriv->Port[0].iBrightness=0;
	pAPriv->Port[0].CCmode=0;
	pAPriv->Port[0].stereo=1;
	pAPriv->Port[0].sap=0;

	OVERLAY_TRANSACTION_START

	  /* below are hardcoded and present room for expansion.. */
	outm(R128_OV0_EXCLUSIVE_HORZ,0); /* disable exclusive mode */
	outm(R128_OV0_VIDEO_KEY_MSK,0xffff);
	outm(R128_OV0_KEY_CNTL,  (5<<4));
	outm(R128_OV0_GRAPHICS_KEY_CLR,pAPriv->Port[0].overlay_key);
	outm(R128_OV0_AUTO_FLIP_CNTL,pAPriv->Port[0].auto_flip_cntl);
	switch(pScrn->depth){
		case 8:
			outm(R128_OV0_GRAPHICS_KEY_MSK,0xff);
			break;
		case 15:
			outm(R128_OV0_GRAPHICS_KEY_MSK,0x7fff);
			break;
		case 16:
			outm(R128_OV0_GRAPHICS_KEY_MSK,0xffff);
			break;
		case 24:
			outm(R128_OV0_GRAPHICS_KEY_MSK,0xffffff);
			break;
		case 32:
			outm(R128_OV0_GRAPHICS_KEY_MSK,0xffffffff);
			break;
		
		}

	outm(R128_OV0_REG_LOAD_CNTL,0x0);
	outm(R128_OV0_DEINTERLACE_PATTERN,0xAAAAA);
	outm(R128_OV0_P1_V_ACCUM_INIT,(2<<20)|1);
	outm(R128_OV0_P23_V_ACCUM_INIT,(2<<20)|1);
	outm(R128_OV0_P1_H_ACCUM_INIT,(3<<28));
	outm(R128_OV0_P23_H_ACCUM_INIT,(2<<28));
	outm(R128_OV0_STEP_BY,1|(1<<8));
	outm(R128_OV0_FILTER_CNTL,0xf); /* use hardcoded coeff's */
	outm(R128_OV0_FILTER_CNTL,0x0); /* use programmable coeff's */
  	outm(R128_OV0_FOUR_TAP_COEF_0   , 0x00002000);
  	outm(R128_OV0_FOUR_TAP_COEF_1   , 0x0D06200D);
  	outm(R128_OV0_FOUR_TAP_COEF_2   , 0x0D0A1C0D);
  	outm(R128_OV0_FOUR_TAP_COEF_3   , 0x0C0E1A0C);
  	outm(R128_OV0_FOUR_TAP_COEF_4   , 0x0C14140C);
	outm(R128_OV0_COLOUR_CNTL,(1<<12)|(1<<20));
	outm(R128_OV0_TEST,0);
	
	outm(R128_OV0_SCALE_CNTL, pAPriv->Port[0].scale_cntl|pAPriv->Port[0].video_format);
	outm(R128_CAP0_TRIG_CNTL,0);
	outm(R128_CAP0_CONFIG,0);
/*	m64_pollscreen(pScrn,pAPriv,0); */
	pAPriv->hw_type=HW_R128; /* all variants are supported */
	}
/*
switch(pATI->Chip){
	case ATI_CHIP_MOBILITY:
		pAPriv->hw_type=HW_RAGEPRO;
		break;
	default:
		pAPriv->hw_type=HW_UNSUPPORTED;
	}*/
return pAPriv;
}

void R128VideoUninit(ScrnInfoPtr pScrn)
{
R128InfoPtr	pATI=R128PTR(pScrn);
PortPrivPtr     pPPriv=(PortPrivPtr)pATI->xvInputPort;

xf86DrvMsgVerb(pScrn->scrnIndex,X_INFO,1,"R128VideoUninit\n");
pPPriv->mute=1;
MSP3430_SetVolume(pPPriv,0);
TDA_mute(pPPriv);
}

void R128VideoReset(ScrnInfoPtr pScrn)
{
R128InfoPtr	pATI=R128PTR(pScrn);
PortPrivPtr     pPPriv=(PortPrivPtr)pATI->xvInputPort;

xf86DrvMsgVerb(pScrn->scrnIndex,X_INFO,1,"R128VideoReset\n");
if(pPPriv==NULL)return;
if(pPPriv->capture_on){
	pPPriv->capture_on=FALSE;
	SWITCH(capture_on(pPPriv));
	}
}

void R128VideoInit(ScreenPtr pScreen)
{
ScrnInfoPtr pScrn=xf86Screens[pScreen->myNum];
R128InfoPtr	pATI=R128PTR(pScrn);
XF86VideoAdaptorRec VAR[2];
XF86VideoAdaptorPtr VARPtrs[2];
DevUnion Private[2];
AdaptorPrivPtr pAPriv;
CARD32 a,b;
int i;

xf86DrvMsgVerb(pScrn->scrnIndex,X_INFO,1,"Initializing Xv driver\n");
pATI->xvInputPort=NULL;

/* pATI->XvEnabled=FALSE; */

if (!(pAPriv = AdaptorPrivInit(pScrn))){
	xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "Xv driver initialization failed\n");
	return;
    	}

if(pAPriv->hw_type==HW_UNSUPPORTED){
	xf86DrvMsgVerb(pScrn->scrnIndex,X_ERROR,1,"Xv: chipset is not supported\n");
	return;
	}

REGION_INIT(pScreen,&(pAPriv->Port[0].prev_cliplist),NullBox,0);
memset(VAR, 0, sizeof(VAR));


xvEncoding   = MAKE_ATOM(XV_ENCODING);
xvHue        = MAKE_ATOM(XV_HUE);
xvSaturation = MAKE_ATOM(XV_SATURATION);
xvBrightness = MAKE_ATOM(XV_BRIGHTNESS);
xvContrast   = MAKE_ATOM(XV_CONTRAST);
xv_freq	     = MAKE_ATOM(XV_FREQ);
xv_volume    = MAKE_ATOM(XV_VOLUME);
xv_mute	     = MAKE_ATOM(XV_MUTE);

xv_format=MAKE_ATOM(XV_FORMAT);
xv_color_key=MAKE_ATOM(XV_COLOR_KEY);
xv_colorkey=MAKE_ATOM(XV_COLORKEY);
xv_paint_cliplist=MAKE_ATOM(XV_PAINT_CLIPLIST);

/* pATI->XvEnabled=TRUE; */

r128_init_vip(pScrn);
pAPriv->Port[0].theatre_num=detect_theatre(pScrn);
if(pAPriv->Port[0].theatre_num>=0){
	theatre_read(&(pAPriv->Port[0]),VIP_VIP_VENDOR_DEVICE_ID,&a);
	theatre_read(&(pAPriv->Port[0]),VIP_VIP_REVISION_ID,&b);
	xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Rage Theatre detected on VIP port %d device id=0x%08x revision=0x%08x\n", pAPriv->Port[0].theatre_num,a,b);
	init_theatre(&(pAPriv->Port[0]));
	}
	else 
	xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Rage Theatre not detected\n");	


/*
pAPriv->Port[0].pI2CBus=xf86CreateI2CBusRec();
pAPriv->Port[0].pI2CBus->BusName="video";
pAPriv->Port[0].pI2CBus->scrnIndex=pScrn->scrnIndex;
pAPriv->Port[0].pI2CBus->HoldTime=10;
pAPriv->Port[0].pI2CBus->DriverPrivate.ptr=pScrn;

pAPriv->Port[0].pI2CBus->I2CAddress=r128_i2c_address;
pAPriv->Port[0].pI2CBus->I2CStop=r128_i2c_stop;
pAPriv->Port[0].pI2CBus->I2CPutByte=r128_i2c_put_byte;
pAPriv->Port[0].pI2CBus->I2CGetByte=r128_i2c_get_byte;
pAPriv->Port[0].pI2CBus->I2CUDelay=r128_i2c_udelay;


if(!xf86I2CBusInit(pAPriv->Port[0].pI2CBus)){
	xf86DrvMsg(pScrn->scrnIndex,X_INFO,"Failed to initialize i2c bus \"video\"\n");
	xf86DestroyI2CBusRec(pAPriv->Port[0].pI2CBus,TRUE,TRUE);
	pAPriv->Port[0].pI2CBus=NULL;
	}
*/

R128_I2CInit(&(pAPriv->Port[0]));

#if 0
xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Scanning i2c bus:\n");
for(i=0;i<0xff;i+=2){
	if(I2CProbeAddress(&(pAPriv->Port[0]), i)){
		xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Found i2c slave device at address 0x%02x\n",i);
		} 
	}
xf86DrvMsg(pScrn->scrnIndex, X_INFO, "      end of scan.\n");
#endif
	
pAPriv->Port[0].tuner_addr=0;
if(I2CProbeAddress(&(pAPriv->Port[0]), 0xc0)){
	pAPriv->Port[0].tuner_addr=0xc0;
	} else
if(I2CProbeAddress(&(pAPriv->Port[0]), 0xc6)){
	pAPriv->Port[0].tuner_addr=0xc6;
	}
if(pAPriv->Port[0].tuner_addr){
	xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Found FI12xx tuner at 0x%02x\n",
		pAPriv->Port[0].tuner_addr);
	}
if(I2CProbeAddress(&(pAPriv->Port[0]), 0x82)){
	xf86DrvMsg(pScrn->scrnIndex, X_WARNING, "Unsupported: Found TDA8425 Stereo Audio Processor\n");
	}
if(I2CProbeAddress(&(pAPriv->Port[0]), 0xB4)){
	xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Found TDA9850 BTSC Stereo+SAP Audio Processor\n");
	pAPriv->Port[0].tda9850=0xB4;	
	} else pAPriv->Port[0].tda9850=0;

if(I2CProbeAddress(&(pAPriv->Port[0]), 0xB6)){
	xf86DrvMsg(pScrn->scrnIndex, X_WARNING, "Unsupported: Found TDA9851 BTSC Economic Stereo Audio Processor\n");
	}


EnableMSP3430(&(pAPriv->Port[0]));
/*MSP3430_SetVolume(&(pAPriv->Port[0]),0x70);*/
if(pAPriv->Port[0].msp3430_addr>0)MSP3430_SetVolume(&(pAPriv->Port[0]),0x0);

pAPriv->Port[0].volume=-1000;
pAPriv->Port[0].mute=1;
TDA_setaudio(&(pAPriv->Port[0]));



pAPriv->Port[0].bt829_mux=2;
bt829_init(&(pAPriv->Port[0]));
if(pAPriv->Port[0].bt829_addr>0)bt829_setmux(&(pAPriv->Port[0]));


for(i=0;i<2;i++){
	Private[i].ptr = (pointer) &pAPriv->Port[i];
	VARPtrs[i] = &VAR[i];

	VAR[i].type = (i == 0) ? XvInputMask : XvOutputMask;
	VAR[i].type |= XvWindowMask | XvVideoMask | XvStillMask | XvImageMask;
	VAR[i].name = "R128";
	VAR[i].flags=VIDEO_OVERLAID_IMAGES|VIDEO_CLIP_TO_VIEWPORT;

	if (i == 0) {		
		if(pAPriv->Port[0].theatre_num>=0){
			VAR[0].pAttributes=r128_attributes_port0_rt;
   		        VAR[0].nAttributes=NUM_R128_ATTRIBUTES_PORT0_RT;
			} else
		if(pAPriv->Port[0].bt829_addr>0){
			VAR[0].pAttributes=r128_attributes_port0_bt;
   		        VAR[0].nAttributes=NUM_R128_ATTRIBUTES_PORT0_BT;
			} else {
			VAR[0].pAttributes=r128_attributes_port0_image;
   		        VAR[0].nAttributes=NUM_R128_ATTRIBUTES_PORT0_IMAGE;
			}
			
	    	VAR[0].nEncodings = sizeof(InputVideoEncodings) / sizeof(InputVideoEncodings[0]);
	    	VAR[0].pEncodings = InputVideoEncodings;
	    	VAR[0].nFormats = sizeof(InputVideoFormats) / sizeof(InputVideoFormats[0]);
	    	VAR[0].pFormats = InputVideoFormats;
		VAR[0].pImages=ImagesG;
		VAR[0].nImages=NUM_IMAGES_G;
		
		} else {
	    	Bool expose = FALSE;

/*
	    xf86GetOptValBool(OutputOptions, OPTION_EXPOSE, &expose);
	    if (expose)	VAR[1].flags = VIDEO_EXPOSE;
*/

	    	VAR[1].nEncodings = sizeof(OutputVideoEncodings) / sizeof(OutputVideoEncodings[0]);
	    	VAR[1].pEncodings = OutputVideoEncodings;
	    	VAR[1].nFormats = sizeof(OutputVideoFormats) / sizeof(OutputVideoFormats[0]);
	    	VAR[1].pFormats = OutputVideoFormats;
		}

	VAR[i].nPorts = 1;
	VAR[i].pPortPrivates = &Private[i];
	VAR[i].PutVideo = R128PutVideo;
	VAR[i].PutStill = R128PutStill;
	VAR[i].GetVideo = R128GetVideo;
	VAR[i].GetStill = R128GetStill;
	VAR[i].StopVideo = R128StopVideo;
	VAR[i].PutImage=R128PutImage;
	VAR[i].ReputImage=R128ReputImage;
	VAR[i].QueryImageAttributes=R128QueryImageAttributesG;
	VAR[i].SetPortAttribute = R128SetPortAttribute;
	VAR[i].GetPortAttribute = R128GetPortAttribute;
	VAR[i].QueryBestSize = R128QueryBestSize;
    }

if (!xf86XVScreenInit(pScreen, VARPtrs, 1)){  /* register only the first, 
						 input port - the output is
						 not ready yet */
	xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "Xv initialization failed\n");
	return;
	}

pATI->xvInputPort=&(pAPriv->Port[0]);

xf86DrvMsgVerb(pScrn->scrnIndex,X_INFO,1,"Xv driver enabled\n");
}
