dnl Configure paths for GATOS
dnl
dnl Made for GTK+ by Owen Taylor     97-11-3
dnl Hacked for GATOS by Daniel Caujolle-Bert 18/04/2000.

dnl AM_PATH_GATOS([MINIMUM-VERSION, [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND ]]])
dnl Test for GATOS, and define GATOS_CFLAGS and GATOS_LIBS
dnl
AC_DEFUN(AM_PATH_GATOS,
[dnl 
dnl Get the cflags and libraries from the gatos-config script
dnl
AC_ARG_WITH(gatos-prefix,[  --with-gatos-prefix=PFX Prefix where GATOS is installed (optional)],
            gatos_config_prefix="$withval", gatos_config_prefix="")
AC_ARG_WITH(gatos-exec-prefix,[  --with-gatos-exec-prefix=PFX                                                                            Exec prefix where GATOS is installed (optional)],
            gatos_config_exec_prefix="$withval", gatos_config_exec_prefix="")
AC_ARG_ENABLE(gatostest, [  --disable-gatostest     Do not try to compile and run a test GATOS program],
		    , enable_gatostest=yes)

  if test x$gatos_config_exec_prefix != x ; then
     gatos_config_args="$gatos_config_args --exec-prefix=$gatos_config_exec_prefix"
     if test x${GATOS_CONFIG+set} != xset ; then
        GATOS_CONFIG=$gatos_config_exec_prefix/bin/gatos-config
     fi
  fi
  if test x$gatos_config_prefix != x ; then
     gatos_config_args="$gatos_config_args --prefix=$gatos_config_prefix"
     if test x${GATOS_CONFIG+set} != xset ; then
        GATOS_CONFIG=$gatos_config_prefix/bin/gatos-config
     fi
  fi

  AC_PATH_PROG(GATOS_CONFIG, gatos-config, no)
  min_gatos_version=ifelse([$1], ,0.99.7,$1)
  AC_MSG_CHECKING(for GATOS version >= $min_gatos_version)
  no_gatos=""
  if test "$GATOS_CONFIG" = "no" ; then
    no_gatos=yes
  else
    GATOS_CFLAGS=`$GATOS_CONFIG $gatos_config_args --cflags`
    GATOS_LIBS=`$GATOS_CONFIG $gatos_config_args --libs`
    gatos_config_major_version=`$GATOS_CONFIG $gatos_config_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\1/'`
    gatos_config_minor_version=`$GATOS_CONFIG $gatos_config_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\2/'`
    gatos_config_micro_version=`$GATOS_CONFIG $gatos_config_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\3/'`
    if test "x$enable_gatostest" = "xyes" ; then
      ac_save_CFLAGS="$CFLAGS"
      ac_save_LIBS="$LIBS"
      CFLAGS="$CFLAGS $GATOS_CFLAGS"
      LIBS="$GATOS_LIBS $LIBS"
dnl
dnl Now check if the installed GATOS is sufficiently new. (Also sanity
dnl checks the results of gatos-config to some extent
dnl
      rm -f conf.gatostest
      AC_TRY_RUN([
#include <gatos.h>
#include <stdio.h>
#include <stdlib.h>

int 
main ()
{
  int major, minor, micro;
   char *tmp_version;

  system ("touch conf.gatostest");

  /* HP/UX 9 (%@#!) writes to sscanf strings */
  tmp_version = strdup("$min_gatos_version");
  if (sscanf(tmp_version, "%d.%d.%d", &major, &minor, &micro) != 3) {
     printf("%s, bad version string\n", "$min_gatos_version");
     exit(1);
   }

  if ((gatos_major_version != $gatos_config_major_version) ||
      (gatos_minor_version != $gatos_config_minor_version) ||
      (gatos_micro_version != $gatos_config_micro_version))
    {
      printf("\n*** 'gatos-config --version' returned %d.%d.%d, but GATOS (%d.%d.%d)\n", 
             $gatos_config_major_version, $gatos_config_minor_version, $gatos_config_micro_version,
             gatos_major_version, gatos_minor_version, gatos_micro_version);
      printf ("*** was found! If gatos-config was correct, then it is best\n");
      printf ("*** to remove the old version of GATOS. You may also be able to fix the error\n");
      printf("*** by modifying your LD_LIBRARY_PATH enviroment variable, or by editing\n");
      printf("*** /etc/ld.so.conf. Make sure you have run ldconfig if that is\n");
      printf("*** required on your system.\n");
      printf("*** If gatos-config was wrong, set the environment variable GATOS_CONFIG\n");
      printf("*** to point to the correct copy of gatos-config, and remove the file config.cache\n");
      printf("*** before re-running configure\n");
    } 
#if defined (GATOS_MAJOR_VERSION) && defined (GATOS_MINOR_VERSION) && defined (GATOS_SUB_VERSION)
  else if ((gatos_major_version != GATOS_MAJOR_VERSION) ||
	   (gatos_minor_version != GATOS_MINOR_VERSION) ||
           (gatos_micro_version != GATOS_SUB_VERSION))
    {
      printf("*** GATOS header files (version %d.%d.%d) do not match\n",
	     GATOS_MAJOR_VERSION, GATOS_MINOR_VERSION, GATOS_SUB_VERSION);
      printf("*** library (version %d.%d.%d)\n",
	     gatos_major_version, gatos_minor_version, gatos_micro_version);
    }
#endif /* defined (GATOS_MAJOR_VERSION) ... */
  else
    {
      if ((gatos_major_version > major) ||
        ((gatos_major_version == major) && (gatos_minor_version > minor)) ||
        ((gatos_major_version == major) && (gatos_minor_version == minor) && (gatos_micro_version >= micro)))
      {
        return 0;
       }
     else
      {
        printf("\n*** An old version of GATOS (%d.%d.%d) was found.\n",
               gatos_major_version, gatos_minor_version, gatos_micro_version);
        printf("*** You need a version of GATOS newer than %d.%d.%d. The latest version of\n",
	       major, minor, micro);
        printf("*** GATOS is always available from:\n");
        printf("***        http://www.core.binghamton.edu/~insomnia/gatos/\n");
        printf("***\n");
        printf("*** If you have already installed a sufficiently new version, this error\n");
        printf("*** probably means that the wrong copy of the gatos-config shell script is\n");
        printf("*** being found. The easiest way to fix this is to remove the old version\n");
        printf("*** of GATOS, but you can also set the GATOS_CONFIG environment to point to the\n");
        printf("*** correct copy of gatos-config. (In this case, you will have to\n");
        printf("*** modify your LD_LIBRARY_PATH enviroment variable, or edit /etc/ld.so.conf\n");
        printf("*** so that the correct libraries are found at run-time))\n");
      }
    }
  return 1;
}
],, no_gatos=yes,[echo $ac_n "cross compiling; assumed OK... $ac_c"])
       CFLAGS="$ac_save_CFLAGS"
       LIBS="$ac_save_LIBS"
     fi
  fi
  if test "x$no_gatos" = x ; then
     AC_MSG_RESULT(yes)
     ifelse([$2], , :, [$2])     
  else
     AC_MSG_RESULT(no)
     if test "$GATOS_CONFIG" = "no" ; then
       echo "*** The gatos-config script installed by GATOS could not be found"
       echo "*** If GATOS was installed in PREFIX, make sure PREFIX/bin is in"
       echo "*** your path, or set the GATOS_CONFIG environment variable to the"
       echo "*** full path to gatos-config."
     else
       if test -f conf.gatostest ; then
        :
       else
          echo "*** Could not run GATOS test program, checking why..."
          CFLAGS="$CFLAGS $GATOS_CFLAGS"
          LIBS="$LIBS $GATOS_LIBS"
          AC_TRY_LINK([
#include <gatos/gatos.h>
#include <stdio.h>
],      [ return ((gatos_major_version) || (gatos_minor_version) || (gatos_micro_version)); ],
        [ echo "*** The test program compiled, but did not run. This usually means"
          echo "*** that the run-time linker is not finding GATOS or finding the wrong"
          echo "*** version of GATOS. If it is not finding GATOS, you'll need to set your"
          echo "*** LD_LIBRARY_PATH environment variable, or edit /etc/ld.so.conf to point"
          echo "*** to the installed location  Also, make sure you have run ldconfig if that"
          echo "*** is required on your system"
	  echo "***"
          echo "*** If you have an old version installed, it is best to remove it, although"
          echo "*** you may also be able to get things to work by modifying LD_LIBRARY_PATH"
          echo "***"],
        [ echo "*** The test program failed to compile or link. See the file config.log for the"
          echo "*** exact error that occured. This usually means GATOS was incorrectly installed"
          echo "*** or that you have moved GATOS since it was installed. In the latter case, you"
          echo "*** may want to edit the gatos-config script: $GATOS_CONFIG" ])
          CFLAGS="$ac_save_CFLAGS"
          LIBS="$ac_save_LIBS"
       fi
     fi
     GATOS_CFLAGS=""
     GATOS_LIBS=""
     ifelse([$3], , :, [$3])
  fi
  AC_SUBST(GATOS_CFLAGS)
  AC_SUBST(GATOS_LIBS)
  rm -f conf.gatostest
])
