/**************************************************************************\
 gatos (General ATI TV and Overlay Software)

  Project Coordinated By Insomnia (Steaphan Greene)
  (insomnia@core.binghamton.edu)

  Copyright (C) 1999 Steaphan Greene, yvind Aabling, Octavian Purdila, 
	Vladimir Dergachev and Christian Lupien.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

\**************************************************************************/

#ifndef GATOS_ATIR128_H
#define GATOS_ATIR128_H 1

#ifdef __cplusplus
extern "C" {
#endif

#ifdef GATOS_ATIR128_C
#define WHERE
#else
#define WHERE	extern
#endif

#include <stdio.h>

#ifdef XY
#undef XY
#endif
#ifdef XYZ
#undef XYZ
#endif

#define XY(Y,X)		\
	( ( (0xFFFF&((u32)(X))) << 16 ) | (0xFFFF&((u32)(Y))) )
#define XYZ(X,Y,Z)	\
	( ( (0xFF&((u32)(X))) << 16 ) | ( (0xFF&((u32)(Y))) << 8 ) | (0xFF&((u32)(Z))) )


#define SURFACE_ARGB1555	3
#define SURFACE_RGB565		4
#define SURFACE_ARGB8888	6
#define SURFACE_YUV9		9
#define SURFACE_YUV12		0xa
#define SURFACE_VYUY422		0xb
#define SURFACE_YVYU422		0xc

#define OV0_SMARTSWITCH			1L  /* 1 - enabled */
#define OV0_PROGRAMMABLE_LOAD_START	0L  /* 1 - USE H_CMP_LOAD field, 0 - active falling edge */

/* OV0_SCALE_CNTL values */
#define DISABLE_R128_VIDEO	0x00FF8B03L
#define ENABLE_R128_VIDEO	\
	(0x61FF0001L| \
	(OV0_PROGRAMMABLE_LOAD_START <<28)| \
	(OV0_SMARTSWITCH<<15)| \
	(surface_type << 8)| \
	((gatos.gamma)?((4-gatos.gamma)<<5):0)| \
	(gatos.cold<<1))

/* capture config constants */
#define BUF_TYPE_FIELD		0
#define BUF_TYPE_ALTERNATING	1
#define BUF_TYPE_FRAME		2


#define BUF_MODE_SINGLE		0
#define BUF_MODE_DOUBLE		1
#define BUF_MODE_TRIPLE		2

/* CAP0_CONFIG values */
#define ENABLE_R128_CAPTURE_WEAVE (0x1C000005L | (BUF_MODE_SINGLE <<7) | (BUF_TYPE_FRAME << 4))
#define ENABLE_R128_CAPTURE_BOB (0x1C000005L | (BUF_MODE_SINGLE <<7) | (BUF_TYPE_ALTERNATING << 4))
//#define ENABLE_R128_CAPTURE	0x1C0000B5L
#define ENABLE_R128_CAPTURE	((video_type==VIDEO_TYPE_WEAVED)? ENABLE_R128_CAPTURE_WEAVE :ENABLE_R128_CAPTURE_BOB)

/* ATI Rage128 function declarations */

int	r128_init(void) ;
int	r128_inita(void) ;
int	r128_initb(void) ;

int	r128_setcolorkey(void) ;
void	r128_enable_video(int) ;
void	r128_enable_capture(int) ;
int	r128_buffer_mode(void) ;
int	r128_setcaptsize(void) ;
int	r128_pollscreen(int) ;
void	r128_setgeometry(void) ;
int	r128_setgamma(void) ;
int	r128_setcold(void) ;

void	r128_yuvcapture(unsigned char*) ;
void	r128_rgbcapture(unsigned char*) ;
void	r128_capture(FILE*, int) ;
void	r128_playback(FILE*, int) ;

void	r128_dumpregs(void) ;

#define CAP0_BUSY	(CAP0_BUF_STATUS & (1L<<30))
#define CAP1_BUSY	(CAP1_BUF_STATUS & (1L<<30))

#ifdef GATOS_ATIR128_C
static int      r128_iscaptoversize(void) ;
#endif

#ifdef __cplusplus
}
#endif

#undef WHERE
#endif
