dnl Configure paths for IBTK
dnl
dnl Made for GTK+ by Owen Taylor     97-11-3
dnl Hacked for IBTK by Daniel Caujolle-Bert 19/04/2000.

dnl AM_PATH_IBTK([MINIMUM-VERSION, [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND ]]])
dnl Test for IBTK, and define IBTK_CXXFLAGS and IBTK_LIBS
dnl
AC_DEFUN(AM_PATH_IBTK,
[dnl 
dnl Get the cflags and libraries from the ibtk-config script
dnl
AC_ARG_WITH(ibtk-prefix,[  --with-ibtk-prefix=PFX  Prefix where IBTK is installed (optional)],
            ibtk_config_prefix="$withval", ibtk_config_prefix="")
AC_ARG_WITH(ibtk-exec-prefix,
   [  --with-ibtk-exec-prefix=PFX
                          Exec prefix where IBTK is installed (optional)],
            ibtk_config_exec_prefix="$withval", ibtk_config_exec_prefix="")
AC_ARG_ENABLE(ibtktest, [  --disable-ibtktest      Do not try to compile and run a test IBTK program],
		    , enable_ibtktest=yes)

  if test x$ibtk_config_exec_prefix != x ; then
     ibtk_config_args="$ibtk_config_args --exec-prefix=$ibtk_config_exec_prefix"
     if test x${IBTK_CONFIG+set} != xset ; then
        IBTK_CONFIG=$ibtk_config_exec_prefix/bin/ibtk-config
     fi
  fi
  if test x$ibtk_config_prefix != x ; then
     ibtk_config_args="$ibtk_config_args --prefix=$ibtk_config_prefix"
     if test x${IBTK_CONFIG+set} != xset ; then
        IBTK_CONFIG=$ibtk_config_prefix/bin/ibtk-config
     fi
  fi

  AC_PATH_PROG(IBTK_CONFIG, ibtk-config, no)
  min_ibtk_version=ifelse([$1], ,0.99.7,$1)
  AC_MSG_CHECKING(for IBTK version >= $min_ibtk_version)
  no_ibtk=""
  if test "$IBTK_CONFIG" = "no" ; then
    no_ibtk=yes
  else
    IBTK_CXXFLAGS=`$IBTK_CONFIG $ibtk_config_args --cflags`
    IBTK_LIBS=`$IBTK_CONFIG $ibtk_config_args --libs`
    ibtk_config_major_version=`$IBTK_CONFIG $ibtk_config_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\1/'`
    ibtk_config_minor_version=`$IBTK_CONFIG $ibtk_config_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\2/'`
    ibtk_config_micro_version=`$IBTK_CONFIG $ibtk_config_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\3/'`
    if test "x$enable_ibtktest" = "xyes" ; then
      ac_save_CXXFLAGS="$CXXFLAGS"
      ac_save_LIBS="$LIBS"
      CXXFLAGS="$CXXFLAGS $IBTK_CXXFLAGS"
      LIBS="$IBTK_LIBS $LIBS"
dnl
dnl Now check if the installed IBTK is sufficiently new. (Also sanity
dnl checks the results of ibtk-config to some extent
dnl
      AC_LANG_CPLUSPLUS()
      rm -f conf.ibtktest
      AC_TRY_RUN([
#include <iwindow.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int 
main ()
{
  int major, minor, micro;
   char *tmp_version;

  system ("touch conf.ibtktest");

  /* HP/UX 9 (%@#!) writes to sscanf strings */
  tmp_version = strdup("$min_ibtk_version");
  if (sscanf(tmp_version, "%d.%d.%d", &major, &minor, &micro) != 3) {
     printf("%s, bad version string\n", "$min_ibtk_version");
     exit(1);
   }

  if ((ibtk_major_version != $ibtk_config_major_version) ||
      (ibtk_minor_version != $ibtk_config_minor_version) ||
      (ibtk_micro_version != $ibtk_config_micro_version))
    {
      printf("\n*** 'ibtk-config --version' returned %d.%d.%d, but IBTK (%d.%d.%d)\n", 
             $ibtk_config_major_version, $ibtk_config_minor_version, $ibtk_config_micro_version,
             ibtk_major_version, ibtk_minor_version, ibtk_micro_version);
      printf ("*** was found! If ibtk-config was correct, then it is best\n");
      printf ("*** to remove the old version of IBTK. You may also be able to fix the error\n");
      printf("*** by modifying your LD_LIBRARY_PATH enviroment variable, or by editing\n");
      printf("*** /etc/ld.so.conf. Make sure you have run ldconfig if that is\n");
      printf("*** required on your system.\n");
      printf("*** If ibtk-config was wrong, set the environment variable IBTK_CONFIG\n");
      printf("*** to point to the correct copy of ibtk-config, and remove the file config.cache\n");
      printf("*** before re-running configure\n");
    } 
#if defined (IBTK_MAJOR_VERSION) && defined (IBTK_MINOR_VERSION) && defined (IBTK_SUB_VERSION)
  else if ((ibtk_major_version != IBTK_MAJOR_VERSION) ||
	   (ibtk_minor_version != IBTK_MINOR_VERSION) ||
           (ibtk_micro_version != IBTK_SUB_VERSION))
    {
      printf("*** IBTK header files (version %d.%d.%d) do not match\n",
	     IBTK_MAJOR_VERSION, IBTK_MINOR_VERSION, IBTK_SUB_VERSION);
      printf("*** library (version %d.%d.%d)\n",
	     ibtk_major_version, ibtk_minor_version, ibtk_micro_version);
    }
#endif /* defined (IBTK_MAJOR_VERSION) ... */
  else
    {
      if ((ibtk_major_version > major) ||
        ((ibtk_major_version == major) && (ibtk_minor_version > minor)) ||
        ((ibtk_major_version == major) && (ibtk_minor_version == minor) && (ibtk_micro_version >= micro)))
      {
        return 0;
       }
     else
      {
        printf("\n*** An old version of IBTK (%d.%d.%d) was found.\n",
               ibtk_major_version, ibtk_minor_version, ibtk_micro_version);
        printf("*** You need a version of IBTK newer than %d.%d.%d. The latest version of\n",
	       major, minor, micro);
        printf("*** IBTK is always available from:\n");
        printf("***        http://www.core.binghamton.edu/~insomnia/ibtk/\n");
        printf("***\n");
        printf("*** If you have already installed a sufficiently new version, this error\n");
        printf("*** probably means that the wrong copy of the ibtk-config shell script is\n");
        printf("*** being found. The easiest way to fix this is to remove the old version\n");
        printf("*** of IBTK, but you can also set the IBTK_CONFIG environment to point to the\n");
        printf("*** correct copy of ibtk-config. (In this case, you will have to\n");
        printf("*** modify your LD_LIBRARY_PATH enviroment variable, or edit /etc/ld.so.conf\n");
        printf("*** so that the correct libraries are found at run-time))\n");
      }
    }
  return 1;
}
],, no_ibtk=yes,[echo $ac_n "cross compiling; assumed OK... $ac_c"])
       CXXFLAGS="$ac_save_CXXFLAGS"
       LIBS="$ac_save_LIBS"
     fi
  fi
  if test "x$no_ibtk" = x ; then
     AC_MSG_RESULT(yes)
     ifelse([$2], , :, [$2])     
  else
     AC_MSG_RESULT(no)
     if test "$IBTK_CONFIG" = "no" ; then
       echo "*** The ibtk-config script installed by IBTK could not be found"
       echo "*** If IBTK was installed in PREFIX, make sure PREFIX/bin is in"
       echo "*** your path, or set the IBTK_CONFIG environment variable to the"
       echo "*** full path to ibtk-config."
     else
       if test -f conf.ibtktest ; then
        :
       else
          echo "*** Could not run IBTK test program, checking why..."
          CXXFLAGS="$CXXFLAGS $IBTK_CXXFLAGS"
          LIBS="$LIBS $IBTK_LIBS"
          AC_TRY_LINK([
#include <iwindow.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
],      [ return ((ibtk_major_version) || (ibtk_minor_version) || (ibtk_micro_version)); ],
        [ echo "*** The test program compiled, but did not run. This usually means"
          echo "*** that the run-time linker is not finding IBTK or finding the wrong"
          echo "*** version of IBTK. If it is not finding IBTK, you'll need to set your"
          echo "*** LD_LIBRARY_PATH environment variable, or edit /etc/ld.so.conf to point"
          echo "*** to the installed location  Also, make sure you have run ldconfig if that"
          echo "*** is required on your system"
	  echo "***"
          echo "*** If you have an old version installed, it is best to remove it, although"
          echo "*** you may also be able to get things to work by modifying LD_LIBRARY_PATH"
          echo "***"],
        [ echo "*** The test program failed to compile or link. See the file config.log for the"
          echo "*** exact error that occured. This usually means IBTK was incorrectly installed"
          echo "*** or that you have moved IBTK since it was installed. In the latter case, you"
          echo "*** may want to edit the ibtk-config script: $IBTK_CONFIG" ])
          CXXFLAGS="$ac_save_CXXFLAGS"
          LIBS="$ac_save_LIBS"
       fi
     fi
     IBTK_CXXFLAGS=""
     IBTK_LIBS=""
     ifelse([$3], , :, [$3])
  fi
  AC_SUBST(IBTK_CXXFLAGS)
  AC_SUBST(IBTK_LIBS)
  rm -f conf.ibtktest
  AC_LANG_C()
])
